''' <summary>
''' Subtraktives Farbmodell 
''' </summary>
''' <remarks>Technische Grundlage fr modernen Vierfarbdruck, Vereinfacht kann es als komplementr zum RGB-Farbmodell verstanden werden.</remarks>
Public Class CMYKColor

#Region " Private Felder "
    Private m_Cyan As Double
    Private m_Magenta As Double
    Private m_Yellow As Double
    Private m_Black As Double
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Konstruktor fr Initialisierung mit fest stehenden CMYK-Werten
    ''' </summary>
    ''' <param name="Cyan">Cyan - Anteil (Wert im Bereich zwischen 0.0 und 1.0)</param>
    ''' <param name="Magenta">Magenta - Anteil (Wert im Bereich zwischen 0.0 und 1.0)</param>
    ''' <param name="Yellow">Yellow - Anteil (Wert im Bereich zwischen 0.0 und 1.0)</param>
    ''' <param name="Black">Schwarz - Anteil (Wert im Bereich zwischen 0.0 und 1.0)</param>
    Public Sub New(ByVal Cyan As Double, ByVal Magenta As Double, ByVal Yellow As Double, ByVal Black As Double)
        m_Cyan = MinMax(Cyan)
        m_Magenta = MinMax(Magenta)
        m_Yellow = MinMax(Yellow)
        m_Black = MinMax(Black)
    End Sub

    ''' <summary>
    ''' Konstruktor fr Initialisierung mit einer Color-Struktur
    ''' </summary>
    ''' <param name="Col">Color-Struktur</param>
    Public Sub New(ByVal Col As Color)
        m_Black = 1.0R

        m_Cyan = CDbl((255 - Col.R)) / 255.0R
        m_Magenta = CDbl((255 - Col.G)) / 255.0R
        m_Yellow = CDbl((255 - Col.B)) / 255.0R

        If m_Black > m_Cyan Then m_Black = m_Cyan
        If m_Black > m_Magenta Then m_Black = m_Magenta
        If m_Black > m_Yellow Then m_Black = m_Yellow
    End Sub

#End Region

#Region " Properties "
    ''' <summary>
    ''' Cyan - Anteil
    ''' </summary>
    ''' <remarks>Wert im Bereich zwischen 0.0 und 1.0</remarks>
    Public Property Cyan() As Double
        Get
            Return m_Cyan
        End Get
        Set(ByVal value As Double)
            m_Cyan = MinMax(value)
        End Set
    End Property


    ''' <summary>
    ''' Magenta - Anteil
    ''' </summary>
    ''' <remarks>Wert im Bereich zwischen 0.0 und 1.0</remarks>
    Public Property Magenta() As Double
        Get
            Return m_Magenta
        End Get
        Set(ByVal value As Double)
            m_Magenta = MinMax(value)
        End Set
    End Property


    ''' <summary>
    ''' Yellow - Anteil
    ''' </summary>
    ''' <remarks>Wert im Bereich zwischen 0.0 und 1.0</remarks>
    Public Property Yellow() As Double
        Get
            Return m_Yellow
        End Get
        Set(ByVal value As Double)
            m_Yellow = MinMax(value)
        End Set
    End Property


    ''' <summary>
    ''' Schwarz - Anteil
    ''' </summary>
    ''' <remarks>Wert im Bereich zwischen 0.0 und 1.0</remarks>
    Public Property Black() As Double
        Get
            Return m_Black
        End Get
        Set(ByVal value As Double)
            m_Black = MinMax(value)
        End Set
    End Property
#End Region

#Region " Methoden "

    ''' <summary>
    ''' Interne Methode, die absichert, dass C, M, Y und K nur einen Wert zwischen 0.0R und 1.0R annehmen knnen
    ''' </summary>
    Private Function MinMax(ByVal Val As Double) As Double
        Return Math.Max(Math.Min(Val, 1.0R), 0.0R)
    End Function

    ''' <summary>
    ''' Gibt eine Color-Struktur zurck
    ''' </summary>
    ''' <returns>Color-Struktur</returns>
    Public Function ToRGB() As Color
        Dim red, green, blue As Integer
        red = CInt(Math.Round(255.0R * (1.0R - m_Cyan)))
        green = CInt(Math.Round(255.0R * (1.0R - m_Magenta)))
        blue = CInt(Math.Round(255.0R * (1.0R - m_Yellow)))

        Return Color.FromArgb(red, green, blue)
    End Function
#End Region

End Class

