Imports System.Drawing.Drawing2D
''' <summary>
''' Eine Klasse, die einen Farbverlauf definiert und Methoden liefert, um den Farbverlauf auf ein Graphics-Object zu zeichnen
''' </summary>
Public Class GradientColor

#Region " Private Felder "
    Private m_Color1 As Color = Color.Gray
    Private m_Color2 As Color = Color.White
    Private m_GradientDirection As GradientDirections = GradientDirections.Vertical
    Private m_Variante As Byte = 0
#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Default-Konstruktor
    ''' </summary>
    ''' <remarks>Erstellt einen Gradienten mit Color1=Gray, Color2=White und vertikaler Ausrichtung</remarks>
    Public Sub New()

    End Sub

    ''' <summary>
    ''' Konstruktor fr fest stehende Parameter
    ''' </summary>
    ''' <param name="Color1">Farbe1</param>
    ''' <param name="Color2">Farbe2</param>
    ''' <param name="GradientDirection">Art des Gradienten</param>
    ''' <param name="Variante">Variante (Ausrichtung) des Gradienten</param>
    Public Sub New(ByVal Color1 As Color, ByVal Color2 As Color, ByVal GradientDirection As GradientDirections, ByVal Variante As Byte)
        m_Color1 = Color1
        m_Color2 = Color2
        m_GradientDirection = GradientDirection
        m_Variante = Variante
    End Sub

    ''' <summary>
    ''' Konstruktor fr Initialisierung mit einem bekannten Farbverlauf
    ''' </summary>
    ''' <param name="BaseShapeColor"></param>
    Public Sub New(ByVal BaseShapeColor As GradientColor)
        m_Color1 = BaseShapeColor.Color1
        m_Color2 = BaseShapeColor.Color2
        m_GradientDirection = BaseShapeColor.GradientDirection
        m_Variante = BaseShapeColor.Variante
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' erste Farbe des Gradienten
    ''' </summary>
    Property Color1() As Color
        Get
            Return m_Color1
        End Get
        Set(ByVal value As Color)
            m_Color1 = value
        End Set
    End Property

    ''' <summary>
    ''' zweite Farbe des Gradienten
    ''' </summary>
    Property Color2() As Color
        Get
            Return m_Color2
        End Get
        Set(ByVal value As Color)
            m_Color2 = value
        End Set
    End Property

    ''' <summary>
    ''' Art des Gradienten
    ''' </summary>
    ReadOnly Property GradientDirection() As GradientDirections
        Get
            Return m_GradientDirection
        End Get
    End Property

    ''' <summary>
    ''' Variante (Ausrichtung) des Gradienten
    ''' </summary>
    ReadOnly Property Variante() As Byte
        Get
            Return m_Variante
        End Get
    End Property

#End Region

#Region " Methoden "
    ''' <summary>
    ''' legt Art und Ausrichtung des Gradienten fest
    ''' </summary>
    Public Sub SetStyle(ByVal GradientDirection As GradientDirections, ByVal Variante As Byte)
        m_Variante = Variante
        m_GradientDirection = GradientDirection
    End Sub
    ''' <summary>
    ''' vergleicht den aktuellen mit einem anderen Gradienten (z.B. zum Sortieren)
    ''' </summary>
    ''' <param name="Col">Gradient, mit dem verglichen werden soll</param>
    Public Function CompareTo(ByVal Col As GradientColor) As Integer
        If Col.Color1.R = m_Color1.R Then
            If Col.Color1.G = m_Color1.G Then
                If Col.Color1.B = m_Color1.B Then
                    If Col.Color2.R = m_Color2.R Then
                        If Col.Color2.G = m_Color2.G Then
                            If Col.Color2.B = m_Color2.B Then
                                If Col.GradientDirection = m_GradientDirection Then
                                    Return Col.Variante.CompareTo(m_Variante)
                                Else
                                    Return Col.GradientDirection.CompareTo(m_GradientDirection)
                                End If
                            Else
                                Return Col.Color2.B.CompareTo(m_Color2.B)
                            End If
                        Else
                            Return Col.Color2.G.CompareTo(m_Color2.G)
                        End If
                    Else
                        Return Col.Color2.R.CompareTo(m_Color2.R)
                    End If

                Else
                    Return Col.Color1.B.CompareTo(m_Color1.B)
                End If
            Else
                Return Col.Color1.G.CompareTo(m_Color1.G)
            End If
        Else
            Return Col.Color1.R.CompareTo(m_Color1.R)
        End If

    End Function

    ''' <summary>
    ''' zeichnet diesen Gradienten in einem festgelegten Rechteck auf eine GDI+-Zeichnungsoberflche
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    ''' <param name="rect">Rechteck, in dem gezeichnet werden soll</param>
    Public Sub Draw(ByVal g As System.Drawing.Graphics, ByVal rect As System.Drawing.Rectangle)
        Dim DrawingRect As New RectangleF(CSng(rect.X) - CSng(0.5), CSng(rect.Y) - CSng(0.5), CSng(rect.Width), CSng(rect.Height))


        Dim P(3) As PointF
        P(0) = New PointF(DrawingRect.Left, DrawingRect.Top)
        P(1) = New PointF(DrawingRect.Right, DrawingRect.Top)
        P(2) = New PointF(DrawingRect.Right, DrawingRect.Bottom)
        P(3) = New PointF(DrawingRect.Left, DrawingRect.Bottom)

        g.SmoothingMode = SmoothingMode.AntiAlias

        Dim Col1, Col2 As Color
        Select Case m_Variante
            Case 0, 2
                Col1 = m_Color1
                Col2 = m_Color2
            Case 1, 3
                Col1 = m_Color2
                Col2 = m_Color1
        End Select

        Select Case m_GradientDirection
            Case GradientDirections.Vertical
                Dim Brsh As New LinearGradientBrush(P(0), P(1), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRectangle(Brsh, DrawingRect)
                Brsh.Dispose()
            Case GradientDirections.Horizontal
                Dim Brsh As New LinearGradientBrush(P(0), P(3), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRectangle(Brsh, DrawingRect)
                Brsh.Dispose()
            Case GradientDirections.DiagonalTop
                Dim Brsh As New LinearGradientBrush(P(0), P(2), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRectangle(Brsh, DrawingRect)
                Brsh.Dispose()
            Case GradientDirections.DiagonalBottom
                Dim Brsh As New LinearGradientBrush(P(3), P(1), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRectangle(Brsh, DrawingRect)
                Brsh.Dispose()
            Case GradientDirections.Corner

                Dim Brsh As New PathGradientBrush(P)
                Brsh.CenterColor = m_Color1
                Dim SurroundColor As Color() = {m_Color2}
                Brsh.SurroundColors = SurroundColor
                Brsh.CenterPoint = P(m_Variante)
                g.FillRectangle(Brsh, DrawingRect)
                Brsh.Dispose()
            Case GradientDirections.Center
                If (m_Variante = 0) OrElse (m_Variante = 1) Then
                    Dim Brsh As New PathGradientBrush(P)
                    Brsh.CenterColor = Col2
                    Dim SurroundColor As Color() = {Col1}
                    Brsh.SurroundColors = SurroundColor
                    g.FillRectangle(Brsh, DrawingRect)
                    Brsh.Dispose()
                Else
                    Dim myPath As New GraphicsPath
                    myPath.AddEllipse(DrawingRect)
                    Dim Brsh As New PathGradientBrush(myPath)
                    Dim SurroundColor As Color() = {Col1}
                    Brsh.SurroundColors = SurroundColor
                    Brsh.CenterColor = Col2
                    g.FillRectangle(New SolidBrush(Col1), DrawingRect)
                    g.FillRectangle(Brsh, DrawingRect)
                    Brsh.Dispose()
                End If
        End Select

        g.SmoothingMode = SmoothingMode.Default
    End Sub


    ''' <summary>
    ''' zeichnet diesen Gradienten in einer Drawing.Region auf ein Graphics-Objekt
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    ''' <param name="reg">Drawing.Region</param>
    Public Sub FillRegion(ByVal g As System.Drawing.Graphics, ByVal reg As System.Drawing.Region)
        Dim DrawingRect As RectangleF = reg.GetBounds(g)

        Dim P(3) As PointF
        P(0) = New PointF(DrawingRect.Left, DrawingRect.Top)
        P(1) = New PointF(DrawingRect.Right, DrawingRect.Top)
        P(2) = New PointF(DrawingRect.Right, DrawingRect.Bottom)
        P(3) = New PointF(DrawingRect.Left, DrawingRect.Bottom)

        g.SmoothingMode = SmoothingMode.AntiAlias

        Dim Col1, Col2 As Color
        Select Case m_Variante
            Case 0, 2
                Col1 = m_Color1
                Col2 = m_Color2
            Case 1, 3
                Col1 = m_Color2
                Col2 = m_Color1
        End Select

        Select Case m_GradientDirection
            Case GradientDirections.Vertical
                Dim Brsh As New LinearGradientBrush(P(0), P(1), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRegion(Brsh, reg)
                Brsh.Dispose()
            Case GradientDirections.Horizontal
                Dim Brsh As New LinearGradientBrush(P(0), P(3), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRegion(Brsh, reg)
                Brsh.Dispose()
            Case GradientDirections.DiagonalTop
                Dim Brsh As New LinearGradientBrush(P(0), P(2), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRegion(Brsh, reg)
                Brsh.Dispose()
            Case GradientDirections.DiagonalBottom
                Dim Brsh As New LinearGradientBrush(P(3), P(1), Col1, Col2)
                If (m_Variante = 2) OrElse (m_Variante = 3) Then
                    Brsh.SetBlendTriangularShape(CSng(0.5))
                End If
                g.FillRegion(Brsh, reg)
                Brsh.Dispose()
            Case GradientDirections.Corner

                Dim Brsh As New PathGradientBrush(P)
                Brsh.CenterColor = m_Color1
                Dim SurroundColor As Color() = {m_Color2}
                Brsh.SurroundColors = SurroundColor
                Brsh.CenterPoint = P(m_Variante)
                g.FillRegion(Brsh, reg)
                Brsh.Dispose()
            Case GradientDirections.Center
                If (m_Variante = 0) OrElse (m_Variante = 1) Then
                    Dim Brsh As New PathGradientBrush(P)
                    Brsh.CenterColor = Col2
                    Dim SurroundColor As Color() = {Col1}
                    Brsh.SurroundColors = SurroundColor
                    g.FillRegion(Brsh, reg)
                    Brsh.Dispose()
                Else
                    Dim myPath As New GraphicsPath
                    myPath.AddEllipse(DrawingRect)
                    Dim Brsh As New PathGradientBrush(myPath)
                    Dim SurroundColor As Color() = {Col1}
                    Brsh.SurroundColors = SurroundColor
                    Brsh.CenterColor = Col2
                    g.FillRegion(New SolidBrush(Col1), reg)
                    g.FillRegion(Brsh, reg)
                    Brsh.Dispose()
                End If
        End Select

        g.SmoothingMode = SmoothingMode.Default
    End Sub
#End Region

End Class
