Imports System.ComponentModel
''' <summary>
''' DropDown-Button, der eine Farbe anzeigt
''' </summary>
Public Class ColorButton
    Inherits Button

#Region " Private Felder "
    Private m_Color As Color = Color.Green
    Private m_ColorBrush As New SolidBrush(Color.Green)
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Wesentliche Eigenschaften auf Start-Werte festlegen
        Me.Size = New Size(72, 56)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Farbe, die der Button anzeigt
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As Color
        Get
            Return m_Color
        End Get
        Set(ByVal Value As Color)
            m_Color = Value
            m_ColorBrush = New SolidBrush(m_Color)
            ' Neu Zeichnen erzwingen
            Invalidate()
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="pevent">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal pevent As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pevent)
        Dim g As Graphics = pevent.Graphics

        Dim r As Rectangle = ClientRectangle

        Dim border As Integer = 4
        Dim right_border As Integer = 15

        Dim BorderRectangle As New Rectangle(r.Left + border, r.Top + border, r.Width - border - right_border - 1, r.Height - border * 2 - 1)
        Dim InnerRectangle As New Rectangle(r.Left + border + 1, r.Top + border + 1, r.Width - border - right_border - 3, r.Height - border * 2 - 3)
        Dim ColorRectangle As New Rectangle(r.Left + border + 2, r.Top + border + 2, r.Width - border - right_border - 5, r.Height - border * 2 - 5)


        'Farb-Rechteck und Rahmen zeichnen
        Dim BlackBrush As SolidBrush = Brushes.Black
        Dim WhiteBrush As SolidBrush = Brushes.White
        g.FillRectangle(BlackBrush, BorderRectangle)
        g.FillRectangle(WhiteBrush, InnerRectangle)
        g.FillRectangle(m_ColorBrush, ColorRectangle)

        Dim pen As Pen = Pens.Black

        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        'Pfeil fr DropDown zeichnen
        Dim p1 As New Point(r.Width - 9, r.Height / 2 - 1)
        Dim p2 As New Point(r.Width - 5, r.Height / 2 - 1)
        g.DrawLine(pen, p1, p2)

        p1 = New Point(r.Width - 8, r.Height / 2)
        p2 = New Point(r.Width - 6, r.Height / 2)
        g.DrawLine(pen, p1, p2)

        p1 = New Point(r.Width - 7, r.Height / 2)
        p2 = New Point(r.Width - 7, r.Height / 2 + 1)
        g.DrawLine(pen, p1, p2)
        g.SmoothingMode = Drawing2D.SmoothingMode.Default

        'senkrechte Linie zeichnen
        pen = SystemPens.ControlDark
        p1 = New Point(r.Width - 12, 4)
        p2 = New Point(r.Width - 12, r.Height - 5)
        g.DrawLine(pen, p1, p2)

        pen = SystemPens.ControlLightLight
        p1 = New Point(r.Width - 11, 4)
        p2 = New Point(r.Width - 11, r.Height - 5)
        g.DrawLine(pen, p1, p2)

    End Sub
#End Region

End Class
