Imports System.ComponentModel
''' <summary>
''' DropDown-Button, der einen Farbgradienten anzeigen kann
''' </summary>
Public Class GradientColorButton
    Inherits Button

#Region " Private Felder "
    Private m_GradientColor As New GradientColor
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        Me.Size = New Size(72, 56)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Farbgradient, den der Button anzeigt
    ''' </summary>
    <Description("Der ausgewhlte Farb-Gradient"), _
    Category("ColorControl")> _
   Public Property GradientColor() As GradientColor
        Get
            Return m_GradientColor
        End Get
        Set(ByVal Value As GradientColor)
            If m_GradientColor.CompareTo(Value) <> 0 Then
                m_GradientColor = Value
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="pevent">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal pevent As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pevent)
        Dim g As Graphics = pevent.Graphics

        Dim r As Rectangle = ClientRectangle

        Dim border As Integer = 4
        Dim right_border As Integer = 15

        Dim BorderRectangle As New Rectangle(r.Left + border, r.Top + border, r.Width - border - right_border - 1, r.Height - border * 2 - 1)
        Dim InnerRectangle As New Rectangle(r.Left + border + 1, r.Top + border + 1, r.Width - border - right_border - 3, r.Height - border * 2 - 3)
        Dim ColorRectangle As New Rectangle(r.Left + border + 2, r.Top + border + 2, r.Width - border - right_border - 5, r.Height - border * 2 - 5)

        'Gradient und Rahmen in einem Rechteck zeichnen
        g.FillRectangle(Brushes.Black, BorderRectangle)
        g.FillRectangle(Brushes.White, InnerRectangle)
        m_GradientColor.Draw(g, ColorRectangle)


        Dim pen As Pen = Pens.Black

        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        'Pfeil zeichnen
        Dim p1 As New Point(r.Width - 9, r.Height / 2 - 1)
        Dim p2 As New Point(r.Width - 5, r.Height / 2 - 1)
        g.DrawLine(pen, p1, p2)

        p1 = New Point(r.Width - 8, r.Height / 2)
        p2 = New Point(r.Width - 6, r.Height / 2)
        g.DrawLine(pen, p1, p2)

        p1 = New Point(r.Width - 7, r.Height / 2)
        p2 = New Point(r.Width - 7, r.Height / 2 + 1)
        g.DrawLine(pen, p1, p2)
        g.SmoothingMode = Drawing2D.SmoothingMode.Default

        'senkrechte Linie zeichnen
        pen = SystemPens.ControlDark
        p1 = New Point(r.Width - 12, 4)
        p2 = New Point(r.Width - 12, r.Height - 5)
        g.DrawLine(pen, p1, p2)

        pen = SystemPens.ControlLightLight
        p1 = New Point(r.Width - 11, 4)
        p2 = New Point(r.Width - 11, r.Height - 5)
        g.DrawLine(pen, p1, p2)

    End Sub
#End Region

End Class
