Imports System.ComponentModel
''' <summary>
''' Dem Adobe-Farbwhler nachgebildetes Control zum Anzeigen und Festlegen von Farben
''' </summary>
Public Class AdobeColorPicker
    Implements IColorControl

#Region " Private Felder / Events "
    Private m_Color As Color
    Private m_DrawStyle As DrawStyles
    Private m_Init As Boolean = True
    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
#End Region

#Region " Konstruktoren "
    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.

        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)

        m_DrawStyle = DrawStyles.Hue
        m_Color = Drawing.Color.Red

        Me.Color2DBox1.Color = m_Color
        Me.Color2DBox1.DrawStyle = m_DrawStyle
        Me.VerticalColorSlider1.Color = m_Color
        Me.VerticalColorSlider1.DrawStyle = m_DrawStyle
        Me.RadioButtonHue.Checked = True
        m_Init = False
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If m_Color.R <> value.R OrElse m_Color.G <> value.G OrElse m_Color.B <> value.B Then
                m_Color = value
                Me.Color2DBox1.Color = m_Color
                Me.VerticalColorSlider1.Color = m_Color
            End If
        End Set
    End Property

    ''' <summary>
    ''' Anzeige-Stil 
    ''' </summary>
    ''' <remarks>Hue fr fixen Farbton, Saturation fr fixe Sttigung, Brightness fr fixe relative Helligkeit, Red fr fixen Rot-Anteil, Green fr gleich fixen Grn-Anteil, Blue fr gleich bleibenden Blau-Anteil</remarks>
    <Description("Der ausgewhlte Anzeige-Stil"), _
   Category("ColorControl")> _
   Property DrawStyle() As DrawStyles
        Get
            Return m_DrawStyle
        End Get
        Set(ByVal value As DrawStyles)
            If m_DrawStyle <> value Then
                m_Init = True
                Select Case value
                    Case DrawStyles.Hue
                        RadioButtonHue.Checked = True
                    Case DrawStyles.Saturation
                        RadioButtonSat.Checked = True
                    Case DrawStyles.Brightness
                        RadioButtonBlack.Checked = True
                    Case DrawStyles.Red
                        RadioButtonRed.Checked = True
                    Case DrawStyles.Green
                        RadioButtonGreen.Checked = True
                    Case DrawStyles.Blue
                        RadioButtonBlue.Checked = True
                End Select
                m_Init = False
                m_DrawStyle = value
                Me.Color2DBox1.DrawStyle = m_DrawStyle
                Me.VerticalColorSlider1.DrawStyle = m_DrawStyle
            End If
        End Set
    End Property
#End Region

#Region " Methoden "


    ''' <summary>
    ''' reagiert auf nderungen in der (quadratischen) Color2DBox (nach Mauseingabe)
    ''' </summary>
    Private Sub Color2DBox1_ColorChanged(ByVal Sender As IColorControl) Handles Color2DBox1.ColorChanged
        If m_Color.R <> Color2DBox1.Color.R OrElse m_Color.G <> Color2DBox1.Color.G OrElse m_Color.B <> Color2DBox1.Color.B Then
            m_Color = Color2DBox1.Color
            Me.VerticalColorSlider1.BaseColor = m_Color
            RaiseEvent ColorChanged(Me)
        End If
    End Sub

    ''' <summary>
    ''' reagiert auf nderungen im (senkrechten) VerticalColorSlider (nach Mauseingabe)
    ''' </summary>
    ''' <param name="Sender"></param>
    Private Sub VerticalColorSlider1_ColorChanged(ByVal Sender As IColorControl) Handles VerticalColorSlider1.ColorChanged
        If m_Color.R <> VerticalColorSlider1.Color.R OrElse m_Color.G <> VerticalColorSlider1.Color.G OrElse m_Color.B <> VerticalColorSlider1.Color.B Then
            m_Color = VerticalColorSlider1.Color
            Me.Color2DBox1.BaseColor = m_Color
            RaiseEvent ColorChanged(Me)
        End If
    End Sub

    ''' <summary>
    ''' ndert den Anzeige-Stil nach Klick auf einen RadioButton
    ''' </summary>
    Private Sub RadioButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonHue.CheckedChanged, RadioButtonSat.CheckedChanged, RadioButtonBlack.CheckedChanged, RadioButtonRed.CheckedChanged, RadioButtonGreen.CheckedChanged, RadioButtonBlue.CheckedChanged
        If m_Init Then Exit Sub
        Dim NewDrawStyle As DrawStyles
        If RadioButtonHue.Checked Then
            NewDrawStyle = DrawStyles.Hue
        ElseIf RadioButtonSat.Checked Then
            NewDrawStyle = DrawStyles.Saturation
        ElseIf RadioButtonBlack.Checked Then
            NewDrawStyle = DrawStyles.Brightness
        ElseIf RadioButtonRed.Checked Then
            NewDrawStyle = DrawStyles.Red
        ElseIf RadioButtonGreen.Checked Then
            NewDrawStyle = DrawStyles.Green
        ElseIf RadioButtonBlue.Checked Then
            NewDrawStyle = DrawStyles.Blue
        End If

        If m_DrawStyle <> NewDrawStyle Then
            m_DrawStyle = NewDrawStyle
            Me.Color2DBox1.DrawStyle = m_DrawStyle
            Me.VerticalColorSlider1.DrawStyle = m_DrawStyle
        End If
    End Sub
#End Region

End Class
