'******************************************************************'
'*****                                                        *****'
'*****     based on Project:  Adobe Color Picker Clone 1      *****'
'*****     web:               www.codeproject.com             *****'
'*****     Original Author:   Danny Blanchard                 *****'
'*****                        - scrabcakes@gmail.com          *****'
'*****     VB Version by:     Thomas Bergner                  *****'
'*****                        - thbergner@t-online.de         *****'
'*****                                                        *****'
'******************************************************************'
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
''' <summary>
''' Dieses Control gehrt zum AdobeColorPicker
''' </summary>

Public Class Color2DBox
    Inherits Control
    Implements IColorControl

#Region " Private Felder / Events "
    ' Bitmap, das den farbigen Hintergrund des Controls (ohne den runden Marker) hlt
    Private m_BMP As Bitmap
    ' True, wenn der Marker mit der Maus ber das Control gezogen wird
    Private m_Dragging As Boolean = False

    ' Anzeige-Stil (Hue, Saturation, Brightness, Red, Green, Blue)
    Private m_DrawStyle As DrawStyles
    ' ausgewhlte Farbe
    Private m_Color As Color = Drawing.Color.Black
    ' Original-HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness
    Private m_OriginHSL As HSLColor

    ' Position fr Farb-Marker
    Private m_MarkerPosition As New Point(-1, -1)

    'Pen zum Zeichnen des Markers
    Private m_MarkerPen As Pen

    ' Event, das nur geliefert wird, wenn die ausgewhlte Farbe von intern (durch die Maus) gendert wird
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
#End Region

#Region " Konstruktoren "
    Public Sub New()
        'Initialisierung wesentlicher Eigenschaften dieses Controls
        Me.MinimumSize = New Size(20, 20)
        Me.Size = New System.Drawing.Size(258, 258)
        m_MarkerPen = New Pen(Drawing.Color.White)
        m_Color = Drawing.Color.Red
        m_DrawStyle = DrawStyles.Hue

        'Marker an richtige Postion setzen
        SetMarkerPosByColor()

        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Anzeige-Stil
    ''' </summary>
    ''' <remarks>Hue, Saturation, Brightness, Red, Green, Blue</remarks>
    WriteOnly Property DrawStyle() As DrawStyles
        Set(ByVal value As DrawStyles)
            If m_DrawStyle <> value Then
                m_DrawStyle = value
                ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
                SetOriginHSL(m_Color, m_DrawStyle)
                ' Gradient-Bitmap auf Nothing setzen
                m_BMP = Nothing
                ' Marker-Position fr aktuelle Farbe neu ermitteln
                SetMarkerPosByColor()
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' ausgewhlte Farbe
    ''' </summary>
    ''' <remarks>bewirkt vollstndiges Neu-Zeichnen des Controls inclusive Marker</remarks>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If m_Color.R <> value.R OrElse m_Color.G <> value.G OrElse m_Color.B <> value.B Then
                ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
                SetOriginHSL(value, m_DrawStyle)
                SetInternColor(value, False)
                ' Gradient-Bitmap auf Nothing setzen
                m_BMP = Nothing
                ' Marker-Position fr aktuelle Farbe neu berechnen
                SetMarkerPosByColor()
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Basis-Farbe
    ''' </summary>
    ''' <remarks>bewirkt Neu-Zeichnen des Controls ohne nderung der Marker-Position</remarks>
    Public WriteOnly Property BaseColor() As System.Drawing.Color
        Set(ByVal value As System.Drawing.Color)
            ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
            SetOriginHSL(value, m_DrawStyle)
            SetInternColor(value, False)
            ' Gradient-Bitmap auf Nothing setzen
            m_BMP = Nothing
            ' Neu Zeichnen erzwingen
            Invalidate()
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' interne Methode, die die Farbe des Markers (schwarz oder weiss) dem ausgewhlten Farb-Wert anpasst und gegebenenfalls das Changed-Event auslst
    ''' </summary>
    ''' <param name="NewValue">neu anzuzeigende Farbe</param>
    ''' <param name="EventsEnabled">legt fest, ob ein Event bei Farbnderung geliefert werden soll</param>
    Private Sub SetInternColor(ByVal NewValue As Color, ByVal EventsEnabled As Boolean)
        If m_Color <> NewValue Then
            m_Color = NewValue
            Dim NewPenColor As Color
            Dim HSL As New HSLColor(m_Color)
            ' Festlegen der Farbe fr den runden Marker
            If (HSL.Lightness < 200.0R / 255.0R) OrElse (((HSL.Hue < 26.0R / 360.0R) OrElse (HSL.Hue > 200.0R / 360.0R)) AndAlso (HSL.Saturation > 70.0R / 255.0R)) Then
                NewPenColor = Drawing.Color.White
            Else
                NewPenColor = Drawing.Color.Black
            End If
            If m_MarkerPen.Color <> NewPenColor Then
                m_MarkerPen = New Pen(NewPenColor)
            End If
            If EventsEnabled Then
                RaiseEvent ColorChanged(Me)
            End If
        End If
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        'm_BMP ist eine Bitmap, das den Hintergrund-Farbgradienten enthlt
        If m_BMP Is Nothing Then
            'Wenn diese Bitmap Nothing ist, wird sie neu erstellt
            m_BMP = New Bitmap(Me.Width, Me.Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim gr As Graphics = Graphics.FromImage(m_BMP)
            DrawBackGround(gr)
        End If

        'zuerst wird die Hintergrund-Bitmap gezeichnet
        e.Graphics.DrawImage(m_BMP, New Rectangle(0, 0, Me.Width, Me.Height))
        'danach der runde Marker
        DrawMarker(e.Graphics)
        'dann der Rahmen dieses Controls
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' zeichnet den runden Marker auf einem Graphics-Objekt
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    Private Sub DrawMarker(ByVal g As Graphics)
        ' AntiAliasing aktivieren
        g.SmoothingMode = SmoothingMode.AntiAlias
        ' Marker zeichnen
        g.DrawEllipse(m_MarkerPen, m_MarkerPosition.X - 5, m_MarkerPosition.Y - 5, 10, 10)
        ' AntiAliasing aktivieren
        g.SmoothingMode = SmoothingMode.Default
    End Sub

    ''' <summary>
    ''' zeichnet den Hintergrund-Farbradienten auf einem Graphics-Objekt
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    Private Sub DrawBackGround(ByVal g As Graphics)
        Select Case m_DrawStyle
            Case DrawStyles.Hue
                Draw_Style_Hue(g)
            Case DrawStyles.Saturation
                Draw_Style_Saturation(g)
            Case DrawStyles.Brightness
                Draw_Style_Luminance(g)
            Case DrawStyles.Red
                Draw_Style_Red(g)
            Case DrawStyles.Green
                Draw_Style_Green(g)
            Case DrawStyles.Blue
                Draw_Style_Blue(g)
        End Select
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Hue-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Hue(ByVal g As Graphics)

        Dim hsl_start As New HSLColor(m_Color)
        Dim hsl_end As New HSLColor(m_Color)
        hsl_start.Saturation = 0.0R
        hsl_end.Saturation = 1.0R

        For i As Integer = 0 To Me.Height - 3

            ' Fr jede horizontale Linie im Control:
            ' Luminanz an dieser Linie berechnen (Hue und Saturation sind konstant)
            hsl_start.Lightness = 1.0R - CDbl(i) / CDbl(Me.Height - 2)

            hsl_end.Lightness = hsl_start.Lightness

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, Me.Width - 2, 1), hsl_start.ToRGB, hsl_end.ToRGB, 0, False)
            g.FillRectangle(br, New Rectangle(1, i + 1, Me.Width - 2, 1))
        Next
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Saturation-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Saturation(ByVal g As Graphics)

        Dim hsl_start As New HSLColor(m_Color)
        Dim hsl_end As New HSLColor(m_Color)

        hsl_start.Lightness = 1.0R
        hsl_end.Lightness = 0.0R

        For i As Integer = 0 To Me.Width - 3

            ' Fr jede vertikale Linie im Control:
            ' Hue an dieser Linie berechnen (Saturation und Luminance sind konstant)
            hsl_start.Hue = CDbl(i) / CDbl(Me.Width - 2)
            hsl_end.Hue = hsl_start.Hue

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, 1, Me.Height - 2), hsl_start.ToRGB, hsl_end.ToRGB, 90, False)
            g.FillRectangle(br, New Rectangle(i + 1, 1, 1, Me.Height - 2))
        Next
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Luminance-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Luminance(ByVal g As Graphics)

        Dim hsl_start As New HSLColor(m_Color)
        Dim hsl_end As New HSLColor(m_Color)
        hsl_start.Saturation = 1.0R
        hsl_end.Saturation = 0.0R

        For i As Integer = 0 To Me.Width - 3

            ' Fr jede vertikale Linie im Control:
            ' Hue an dieser Linie berechnen (Saturation und Luminance sind konstant)
            hsl_start.Hue = CDbl(i) / CDbl(Me.Width - 2)
            hsl_end.Hue = hsl_start.Hue

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, 1, Me.Height - 2), hsl_start.ToRGB, hsl_end.ToRGB, 90, False)
            g.FillRectangle(br, New Rectangle(i + 1, 1, 1, Me.Height - 2))
        Next
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Red-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Red(ByVal g As Graphics)

        Dim red As Integer = m_Color.R

        For i As Integer = 0 To Me.Height - 3

            ' Fr jede horizontale Linie im Control:
            ' Green an dieser Linie berechnen (Red und Blue sind konstant)
            Dim green As Integer = CInt(Math.Round(255.0R - (255.0R * CDbl(i) / CDbl(Me.Height - 2))))

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, Me.Width - 2, 1), Drawing.Color.FromArgb(red, green, 0), Drawing.Color.FromArgb(red, green, 255), 0, False)
            g.FillRectangle(br, New Rectangle(1, i + 1, Me.Width - 2, 1))
        Next
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Green-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Green(ByVal g As Graphics)

        Dim green As Integer = m_Color.G

        For i As Integer = 0 To Me.Height - 3

            ' Fr jede horizontale Linie im Control:
            ' Red an dieser Linie berechnen (Green und Blue sind konstant)
            Dim red As Integer = CInt(Math.Round(255.0R - (255.0R * CDbl(i) / CDbl(Me.Height - 2))))

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, Me.Width - 2, 1), Drawing.Color.FromArgb(red, green, 0), Drawing.Color.FromArgb(red, green, 255), 0, False)
            g.FillRectangle(br, New Rectangle(1, i + 1, Me.Width - 2, 1))
        Next
    End Sub

    ''' <summary>
    ''' Zeichnet einen Farbgradienten fr fest stehenden Blue-Wert auf einem Graphics-Objekt
    ''' </summary>
    Private Sub Draw_Style_Blue(ByVal g As Graphics)

        Dim blue As Integer = m_Color.B

        For i As Integer = 0 To Me.Height - 3

            ' Fr jede horizontale Linie im Control:
            ' Green an dieser Linie berechnen (Red und Blue sind konstant)
            Dim green As Integer = CInt(Math.Round(255.0R - (255.0R * CDbl(i) / CDbl(Me.Height - 2))))

            Dim br As New LinearGradientBrush(New Rectangle(1, 1, Me.Width - 2, 1), Drawing.Color.FromArgb(0, green, blue), Drawing.Color.FromArgb(255, green, blue), 0, False)
            g.FillRectangle(br, New Rectangle(1, i + 1, Me.Width - 2, 1))
        Next
    End Sub

    ''' <summary>
    ''' liefert die angezeigte Farbe fr eine x, y - Position im Control
    ''' </summary>
    Private Function GetColor(ByVal x As Integer, ByVal y As Integer) As Color

        Select Case m_DrawStyle
            Case DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness
                Dim TmpHSL As New HSLColor(m_OriginHSL)
                If m_DrawStyle = DrawStyles.Hue Then
                    TmpHSL.Saturation = CDbl(x) / CDbl(Me.Width - 2)
                    TmpHSL.Lightness = 1.0R - CDbl(y) / CDbl(Me.Height - 2)
                ElseIf m_DrawStyle = DrawStyles.Saturation Then
                    TmpHSL.Hue = CDbl(x) / CDbl(Me.Width - 2)
                    TmpHSL.Lightness = 1.0R - CDbl(y) / CDbl(Me.Height - 2)
                ElseIf m_DrawStyle = DrawStyles.Brightness Then
                    TmpHSL.Hue = CDbl(x) / CDbl(Me.Width - 2)
                    TmpHSL.Saturation = 1.0R - CDbl(y) / CDbl(Me.Height - 2)
                End If
                Return TmpHSL.ToRGB
            Case Else
                If m_DrawStyle = DrawStyles.Red Then
                    Return Drawing.Color.FromArgb(m_Color.R, CInt(Math.Round(255.0R * (1.0R - CDbl(y) / CDbl(Me.Height - 2)))), CInt(Math.Round(255.0R * CDbl(x) / CDbl(Me.Width - 2))))
                ElseIf m_DrawStyle = DrawStyles.Green Then
                    Return Drawing.Color.FromArgb(CInt(Math.Round(255.0R * (1.0R - CDbl(y) / CDbl(Me.Height - 2)))), m_Color.G, CInt(Math.Round(255.0R * CDbl(x) / CDbl(Me.Width - 2))))
                ElseIf m_DrawStyle = DrawStyles.Blue Then
                    Return Drawing.Color.FromArgb(CInt(Math.Round(255.0R * CDbl(x) / CDbl(Me.Width - 2))), CInt(Math.Round(255.0R * (1.0R - CDbl(y) / CDbl(Me.Height - 2)))), m_Color.B)
                End If
        End Select

    End Function

    ''' <summary>
    ''' legt die Position des Markers fr eine ausgewhlte Farbe fest
    ''' </summary>
    Private Sub SetMarkerPosByColor()

        Dim MarkerX, MarkerY As Double
        Select Case m_DrawStyle
            Case DrawStyles.Hue
                Dim HSL As New HSLColor(m_Color)
                MarkerX = CDbl(Me.Width - 2) * HSL.Saturation
                MarkerY = CDbl(Me.Height - 2) * (1.0R - HSL.Lightness)

            Case DrawStyles.Saturation
                Dim HSL As New HSLColor(m_Color)
                MarkerX = CDbl(Me.Width - 2) * HSL.Hue
                MarkerY = CDbl(Me.Height - 2) * (1.0R - HSL.Lightness)

            Case DrawStyles.Brightness
                Dim HSL As New HSLColor(m_Color)
                MarkerX = CDbl(Me.Width - 2) * HSL.Hue
                MarkerY = CDbl(Me.Height - 2) * (1.0R - HSL.Saturation)

            Case DrawStyles.Red
                MarkerX = CDbl(Me.Width - 2) * CDbl(m_Color.B) / 255.0R
                MarkerY = CDbl(Me.Height - 2) * (1.0R - CDbl(m_Color.G) / 255.0R)

            Case DrawStyles.Green
                MarkerX = CDbl(Me.Width - 2) * CDbl(m_Color.B) / 255.0R
                MarkerY = CDbl(Me.Height - 2) * (1.0R - CDbl(m_Color.R) / 255.0R)

            Case DrawStyles.Blue
                MarkerX = CDbl(Me.Width - 2) * CDbl(m_Color.R) / 255.0R
                MarkerY = CDbl(Me.Height - 2) * (1.0R - CDbl(m_Color.G) / 255.0R)
        End Select

        m_MarkerPosition = New Point(CInt(Math.Round(MarkerX)), CInt(Math.Round(MarkerY)))
    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Gradient-Bitmap auf Nothing setzen
        m_BMP = Nothing
        ' Marker-Position fr aktuelle Farbe neu ermitteln
        SetMarkerPosByColor()
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben der MouseDown-Methode, um den Marker zu bewegen
    ''' </summary>
    Private Sub Color2DBox_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        ' nur wenn linker Mouse-Button gedrckt ist
        If e.Button <> MouseButtons.Left Then Return
        ' Dragging-Flag auf True setzen
        m_Dragging = True
        ' Marker zur Maus-Position bewegen
        MoveMarkerTo(e.Location)
    End Sub

    ''' <summary>
    ''' berschreiben der MouseMove-Methode, um den Marker zu bewegen
    ''' </summary>
    Private Sub Color2DBox_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        ' Nur wenn Dragging-Flag True ist
        If Not m_Dragging Then Return
        ' Marker zur Maus-Position bewegen
        MoveMarkerTo(e.Location)
    End Sub

    ''' <summary>
    ''' berschreiben der MouseUp-Methode, die Marker-Bewegung zu beenden
    ''' </summary>
    Private Sub Color2DBox_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        ' nur wenn linker Mouse-Button gedrckt ist
        If e.Button <> MouseButtons.Left Then Return
        ' Nur wenn Dragging-Flag True ist
        If Not m_Dragging Then Return
        ' Marker zur Maus-Position bewegen
        MoveMarkerTo(e.Location)
        ' Dragging-Flag auf False setzen
        m_Dragging = False
    End Sub

    ''' <summary>
    ''' Interne Methode, die nach Bewegung des Markers zu einem beliebigen Punkt innerhalb des Controls (nach Mouse-Aktion) eine nderung der Anzeige und gegebenenfalls Auslsen des Changed-Events bewirkt
    ''' </summary>
    Private Sub MoveMarkerTo(ByVal MousePosition As Point)
        Dim x As Integer = MousePosition.X - 1
        Dim y As Integer = MousePosition.Y - 1
        ' x darf nur zwischen 0 und Width - 2 liegen
        x = Math.Min(Math.Max(x, 0), Me.Width - 2)

        ' y darf nur zwischen 0 und Height - 2 liegen
        y = Math.Min(Math.Max(y, 0), Me.Height - 2)

        ' wenn die neue Maus-Position gleich der vorherigen ist, ist nichts weiter zu tun
        If x = m_MarkerPosition.X AndAlso y = m_MarkerPosition.Y Then Return

        ' bisheriges Marker-Rechteck ermitteln
        Dim OldRect As New Rectangle(m_MarkerPosition.X - 10, m_MarkerPosition.Y - 10, 20, 20)
        ' neue Marker-Position festlegen
        m_MarkerPosition = New Point(x, y)
        ' neues Marker-Rechteck ermitteln
        Dim NewRect As New Rectangle(m_MarkerPosition.X - 10, m_MarkerPosition.Y - 10, 20, 20)

        ' Neue Farbe fr Mauskoordinaten ermitteln und nderungs-Event liefern
        SetInternColor(GetColor(x, y), True)

        'Neu-Zeichnen nur fr altes und neues Marker-Rechteck erzwingen
        Invalidate(Rectangle.Union(OldRect, NewRect))
    End Sub

    ''' <summary>
    ''' legt bei nderung der Color-Property oder von DrawStyle die Original_HSL-Farbe fest
    ''' </summary>
    Private Sub SetOriginHSL(ByVal Col As Color, ByVal DrawStyle As DrawStyles)
        If DrawStyle = DrawStyles.Blue OrElse DrawStyle = DrawStyles.Green OrElse DrawStyle = DrawStyles.Red Then Exit Sub
        m_OriginHSL = New HSLColor(Col)
    End Sub
#End Region

End Class
