'******************************************************************'
'*****                                                        *****'
'*****     based on Project:  Adobe Color Picker Clone 1      *****'
'*****     web:               www.codeproject.com             *****'
'*****     Original Author:   Danny Blanchard                 *****'
'*****                        - scrabcakes@gmail.com          *****'
'*****     VB Version by:     Thomas Bergner                  *****'
'*****                        - thbergner@t-online.de         *****'
'*****                                                        *****'
'******************************************************************'
Imports System.ComponentModel
''' <summary>
''' Dieses Control gehrt zum AdobeColorPicker
''' </summary>
Public Class VerticalColorSlider
    Inherits Control
    Implements IColorControl

#Region " Private Felder / Events "
    ' Bitmap, das den farbigen Hintergrund des Controls (ohne den Marker) hlt
    Private m_BMP As Bitmap

    ' Position fr Farb-Marker
    Private m_MarkerY As Integer = 0

    ' True, wenn der Marker mit der Maus ber das Control gezogen wird
    Private m_Dragging As Boolean = False

    ' Anzeige-Stil (Hue, Saturation, Brightness, Red, Green, Blue)
    Private m_DrawStyle As DrawStyles
    ' ausgewhlte Farbe
    Private m_Color As Color
    ' Original-HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness
    Private m_OriginHSL As HSLColor

    ' Pen zum Zeichnen des Markers
    Private m_SliderPen As Pen
    ' Brush zum Zeichnen des Markers
    Private m_SliderBrush As SolidBrush
    ' Event, das nur geliefert wird, wenn die ausgewhlte Farbe von intern (durch die Maus) gendert wird
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        Me.MinimumSize = New Size(20, 20)
        Me.Size = New System.Drawing.Size(40, 264)

        m_SliderPen = New Pen(Drawing.Color.FromArgb(116, 114, 106))
        m_SliderBrush = Brushes.White
        m_Color = Drawing.Color.Red
        m_DrawStyle = DrawStyles.Hue

        'Marker an richtige Postion setzen
        SetMarkerPosByColor()

        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Anzeige-Stil
    ''' </summary>
    ''' <remarks>Hue, Saturation, Brightness, Red, Green, Blue</remarks>
    WriteOnly Property DrawStyle() As DrawStyles
        Set(ByVal value As DrawStyles)
            If m_DrawStyle <> value Then
                m_DrawStyle = value
                ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
                SetOriginHSL(m_Color, m_DrawStyle)
                ' Gradient-Bitmap auf Nothing setzen
                m_BMP = Nothing
                ' Marker-Position neu festlegen
                SetMarkerPosByColor()
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' ausgewhlte Farbe
    ''' </summary>
    ''' <remarks>bewirkt vollstndiges Neu-Zeichnen des Controls inclusive Marker</remarks>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If m_Color <> value Then
                m_Color = value
                ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
                SetOriginHSL(value, m_DrawStyle)
                ' Gradient-Bitmap auf Nothing setzen
                m_BMP = Nothing
                ' Marker-Position fr Farbe neu ermitteln
                SetMarkerPosByColor()
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Basis-Farbe
    ''' </summary>
    ''' <remarks>bewirkt Neu-Zeichnen des Controls ohne nderung der Marker-Position</remarks>
    Public WriteOnly Property BaseColor() As System.Drawing.Color
        Set(ByVal value As System.Drawing.Color)
            ' Original_HSL-Farbe fr DrawStyles.Hue, DrawStyles.Saturation, DrawStyles.Brightness ermitteln
            SetOriginHSL(value, m_DrawStyle)
            m_Color = value
            ' Fr DrawStyle = Hue bleibt das Hintergrund-Bitmap das gleiche,
            ' ansonsten Gradient-Bitmap fr auf Nothing setzen
            If m_DrawStyle <> DrawStyles.Hue Then m_BMP = Nothing
            ' Neu Zeichnen erzwingen
            Invalidate()
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        'm_BMP ist eine Bitmap, das den Hintergrund-Farbgradienten enthlt
        If m_BMP Is Nothing Then
            ' Wenn diese Bitmap Nothing ist, wird sie neu erstellt
            m_BMP = New Bitmap(Me.Width, Me.Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim gr As Graphics = Graphics.FromImage(m_BMP)
            DrawBackGround(gr)
        End If

        ' zuerst wird die Hintergrund-Bitmap gezeichnet
        e.Graphics.DrawImage(m_BMP, New Rectangle(0, 0, Me.Width, Me.Height))
        ' danach der Rahmen dieses Controls
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(10, 3, Me.Width - 21, Me.Height - 7))
        ' dann der Marker
        DrawSlider(e.Graphics)
    End Sub

    ''' <summary>
    ''' zeichnet den Marker des Controls
    ''' </summary>
    Private Sub DrawSlider(ByVal g As Graphics)

        ' AntiAliasing aktivieren
        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        ' linkes Dreieck
        Dim arrow(6) As Point
        arrow(0) = New Point(1, m_MarkerY)
        arrow(1) = New Point(3, m_MarkerY)
        arrow(2) = New Point(7, m_MarkerY + 4)
        arrow(3) = New Point(3, m_MarkerY + 8)
        arrow(4) = New Point(1, m_MarkerY + 8)
        arrow(5) = New Point(0, m_MarkerY + 7)
        arrow(6) = New Point(0, m_MarkerY + 1)
        ' mit weissem Brush fllen
        g.FillPolygon(m_SliderBrush, arrow)
        ' Rand mit grauem Pen zeichnen
        g.DrawPolygon(m_SliderPen, arrow)

        ' rechtes Dreieck
        arrow(0) = New Point(Me.Width - 2, m_MarkerY)
        arrow(1) = New Point(Me.Width - 4, m_MarkerY)
        arrow(2) = New Point(Me.Width - 8, m_MarkerY + 4)
        arrow(3) = New Point(Me.Width - 4, m_MarkerY + 8)
        arrow(4) = New Point(Me.Width - 2, m_MarkerY + 8)
        arrow(5) = New Point(Me.Width - 1, m_MarkerY + 7)
        arrow(6) = New Point(Me.Width - 1, m_MarkerY + 1)

        ' mit weissem Brush fllen
        g.FillPolygon(m_SliderBrush, arrow)
        ' Rand mit grauem Pen zeichnen
        g.DrawPolygon(m_SliderPen, arrow)

        ' AntiAliasing deaktivieren
        g.SmoothingMode = Drawing2D.SmoothingMode.Default

    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Gradient-Bitmap auf Nothing setzen
        m_BMP = Nothing
        ' Marker-Position fr aktuelle Farbe neu berechnen
        SetMarkerPosByColor()
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' zeichnet den Hintergrund-Farbradienten auf einem Graphics-Objekt
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    Private Sub DrawBackGround(ByVal g As Graphics)
        Select Case m_DrawStyle
            Case DrawStyles.Hue
                Draw_Style_Hue(g)
            Case DrawStyles.Saturation
                Draw_Style_Saturation(g)
            Case DrawStyles.Brightness
                Draw_Style_Luminance(g)
            Case DrawStyles.Red
                Draw_Style_Red(g)
            Case DrawStyles.Green
                Draw_Style_Green(g)
            Case DrawStyles.Blue
                Draw_Style_Blue(g)
        End Select
    End Sub

    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Hue-Werte (0 bis 360) fr fest stehende Luminance- und Saturation-Werte
    ''' </summary>
    Private Sub Draw_Style_Hue(ByVal g As Graphics)

        Dim HSL As New HSLColor(m_Color)
        HSL.Saturation = 1
        HSL.Lightness = 1
        For i As Integer = 0 To Me.Height - 9

            HSL.Hue = 1.0R - CDbl(i) / CDbl(Me.Height - 8)
            Dim pen As New Pen(HSL.ToRGB)
            g.DrawLine(pen, 11, i + 4, Me.Width - 11, i + 4)
            pen.Dispose()
        Next
    End Sub


    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Saturation-Werte (0 bis 100%) fr fest stehende Hue- und Luminance-Werte
    ''' </summary>
    Private Sub Draw_Style_Saturation(ByVal g As Graphics)

        Dim HSL As New HSLColor(m_Color)
        Dim StartCol, EndCol As Color
        HSL.Saturation = 1.0R

        StartCol = HSL.ToRGB
        HSL.Saturation = 0.0R
        EndCol = HSL.ToRGB

        Dim Rect As New Rectangle(11, 4, Me.Width - 22, Me.Height - 8)
        Dim Brsh As New Drawing2D.LinearGradientBrush(Rect, StartCol, EndCol, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(Brsh, Rect)
        Brsh.Dispose()

    End Sub


    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Luminance-Werte (0 to 100%) fr fest stehende Hue- und Saturation-Werte
    ''' </summary>
    Private Sub Draw_Style_Luminance(ByVal g As Graphics)

        Dim HSL As New HSLColor(m_Color)
        Dim StartCol, EndCol As Color
        HSL.Lightness = 1.0R
        StartCol = HSL.ToRGB
        HSL.Lightness = 0.0R
        EndCol = HSL.ToRGB
        Dim Rect As New Rectangle(11, 4, Me.Width - 22, Me.Height - 8)
        Dim Brsh As New Drawing2D.LinearGradientBrush(Rect, StartCol, EndCol, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(Brsh, Rect)
        Brsh.Dispose()

    End Sub


    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Red-Werte (0 to 255) fr fest stehende Green- und Blue-Werte
    ''' </summary>
    Private Sub Draw_Style_Red(ByVal g As Graphics)
        Dim StartCol, EndCol As Color
        StartCol = Color.FromArgb(255, m_Color.G, m_Color.B)
        EndCol = Color.FromArgb(0, m_Color.G, m_Color.B)

        Dim Rect As New Rectangle(11, 4, Me.Width - 22, Me.Height - 8)
        Dim Brsh As New Drawing2D.LinearGradientBrush(Rect, StartCol, EndCol, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(Brsh, Rect)
        Brsh.Dispose()

    End Sub


    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Green-Werte (0 to 255) fr fest stehende Red- und Blue-Werte
    ''' </summary>
    Private Sub Draw_Style_Green(ByVal g As Graphics)
        Dim StartCol, EndCol As Color
        StartCol = Color.FromArgb(m_Color.R, 255, m_Color.B)
        EndCol = Color.FromArgb(m_Color.R, 0, m_Color.B)
        Dim Rect As New Rectangle(11, 4, Me.Width - 22, Me.Height - 8)
        Dim Brsh As New Drawing2D.LinearGradientBrush(Rect, StartCol, EndCol, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(Brsh, Rect)
        Brsh.Dispose()

    End Sub


    ''' <summary>
    ''' Zeichnet den Farbgradienten des Controls fr alle Blue-Werte (0 to 255) fr fest stehende Red- und Green-Werte
    ''' </summary>
    Private Sub Draw_Style_Blue(ByVal g As Graphics)
        Dim StartCol, EndCol As Color
        StartCol = Color.FromArgb(m_Color.R, m_Color.G, 255)
        EndCol = Color.FromArgb(m_Color.R, m_Color.G, 0)
        Dim Rect As New Rectangle(11, 4, Me.Width - 22, Me.Height - 8)
        Dim Brsh As New Drawing2D.LinearGradientBrush(Rect, StartCol, EndCol, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(Brsh, Rect)
        Brsh.Dispose()
    End Sub

    ''' <summary>
    ''' legt die Position des Markers fr eine ausgewhlte Farbe fest
    ''' </summary>
    Private Sub SetMarkerPosByColor()
        Select Case m_DrawStyle
            Case DrawStyles.Hue
                Dim HSL As New HSLColor(m_Color)
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * HSL.Hue))

            Case DrawStyles.Saturation
                Dim HSL As New HSLColor(m_Color)
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * HSL.Saturation))

            Case DrawStyles.Brightness
                Dim HSL As New HSLColor(m_Color)
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * HSL.Lightness))

            Case DrawStyles.Red
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * CDbl(m_Color.R) / 255.0R))

            Case DrawStyles.Green
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * CDbl(m_Color.G) / 255.0R))

            Case DrawStyles.Blue
                m_MarkerY = (Me.Height - 8) - CInt(Math.Round(CDbl(Me.Height - 8) * CDbl(m_Color.B) / 255.0R))

        End Select

        If m_MarkerY < 0 Then
            m_MarkerY = 0
        End If
        If m_MarkerY > Me.Height - 9 Then
            m_MarkerY = Me.Height - 9
        End If
    End Sub

    ''' <summary>
    ''' liefert die angezeigte Farbe fr eine y - Position im Control
    ''' </summary>
    Private Function GetColor(ByVal y As Integer) As Color
        Select Case m_DrawStyle
            Case DrawStyles.Hue
                Dim HSL As New HSLColor(m_OriginHSL)
                HSL.Hue = 1.0R - CDbl(m_MarkerY) / CDbl(Me.Height - 9)
                Return HSL.ToRGB

            Case DrawStyles.Saturation
                Dim HSL As New HSLColor(m_OriginHSL)
                HSL.Saturation = 1.0R - CDbl(m_MarkerY) / CDbl(Me.Height - 9)
                Return HSL.ToRGB

            Case DrawStyles.Brightness
                Dim HSL As New HSLColor(m_OriginHSL)
                HSL.Lightness = 1.0R - CDbl(m_MarkerY) / CDbl(Me.Height - 9)
                Return HSL.ToRGB

            Case DrawStyles.Red
                Return Drawing.Color.FromArgb(255 - CInt(Math.Round(255.0R * CDbl(m_MarkerY) / CDbl(Me.Height - 9))), m_Color.G, m_Color.B)

            Case DrawStyles.Green
                Return Drawing.Color.FromArgb(m_Color.R, 255 - CInt(Math.Round(255.0R * CDbl(m_MarkerY) / CDbl(Me.Height - 9))), m_Color.B)

            Case DrawStyles.Blue
                Return Drawing.Color.FromArgb(m_Color.R, m_Color.G, 255 - CInt(Math.Round(255.0R * CDbl(m_MarkerY) / CDbl(Me.Height - 9))))

        End Select
    End Function

    ''' <summary>
    ''' berschreiben der MouseDown-Methode, um den Marker zu bewegen
    ''' </summary>
    Private Sub VerticalColorSlider_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If e.Button <> MouseButtons.Left Then Return
        m_Dragging = True
        MoveMarkerTo(e.Y)
    End Sub

    ''' <summary>
    ''' berschreiben der MouseMove-Methode, um den Marker zu bewegen
    ''' </summary>
    Private Sub VerticalColorSlider_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If Not m_Dragging Then Return
        MoveMarkerTo(e.Y)
    End Sub

    ''' <summary>
    ''' berschreiben der MouseUp-Methode, die Marker-Bewegung zu beenden
    ''' </summary>
    Private Sub VerticalColorSlider_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If e.Button <> MouseButtons.Left Then Return
        m_Dragging = False
        MoveMarkerTo(e.Y)
    End Sub

    ''' <summary>
    ''' Interne Methode, die nach Bewegung des Markers zu einem beliebigen Punkt innerhalb des Controls (nach Mouse-Aktion) eine nderung der Anzeige und gegebenenfalls Auslsen des Changed-Events bewirkt
    ''' </summary>
    Private Sub MoveMarkerTo(ByVal MouseY As Integer)
        Dim y As Integer = MouseY - 4
        ' Slider Position berechnen

        ' y darf nur zwischen 0 und Height - 2 liegen
        y = Math.Min(Math.Max(y, 0), Me.Height - 9)

        If y = m_MarkerY Then Return
        ' Wenn der Slider nicht bewegt wurde, ist nichts weiter zu tun

        ' altes Marker-Rechteck
        Dim OldRect As New Rectangle(0, m_MarkerY - 10, Me.Width, 20)
        ' Marker-Position festlegen
        m_MarkerY = y
        ' neues Marker-Rechteck
        Dim NewRect As New Rectangle(0, m_MarkerY - 10, Me.Width, 20)

        ' neue Farbe festlegen
        Dim NewColor As Color = GetColor(y)
        If NewColor <> m_Color Then
            m_Color = NewColor
            'Changed-Event liefern
            RaiseEvent ColorChanged(Me)
        End If

        'Nur Neuzeichnen des Union-Rechtecks erzwingen
        Invalidate(Rectangle.Union(OldRect, NewRect))
    End Sub

    ''' <summary>
    ''' legt bei nderung der Color-Property oder von DrawStyle die Original_HSL-Farbe fest
    ''' </summary>
    Private Sub SetOriginHSL(ByVal Col As Color, ByVal DrawStyle As DrawStyles)
        If DrawStyle = DrawStyles.Blue OrElse DrawStyle = DrawStyles.Green OrElse DrawStyle = DrawStyles.Red Then Exit Sub
        m_OriginHSL = New HSLColor(Col)
    End Sub
#End Region

End Class
