Imports System.ComponentModel
''' <summary>
''' Ein Control, das Start- und einen aktuellen Farb-Wert anzeigt 
''' und gegebenenfalls den aktuellen auf den Start-Wert zurck setzen kann
''' </summary>
Public Class BiColorBox
    Inherits Control
    Implements IColorControl

#Region " Private Felder / Events"

    ' Brush mit der aktiven Farbe
    Private m_ActiveBrush As SolidBrush
    ' Brush mit der Start-Farbe
    Private m_StartBrush As SolidBrush

    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        Me.MinimumSize = New Size(50, 50)
        m_ActiveBrush = New SolidBrush(Color.Red)
        m_StartBrush = New SolidBrush(Color.Red)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Color-Struktur, aktuell ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_ActiveBrush.Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If m_ActiveBrush.Color <> value Then
                m_ActiveBrush.Color = value
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property
    ''' <summary>
    ''' Color-Struktur mit der die Startfarbe festgelegt wird
    ''' </summary>
    <Description("Die angezeigte Start-Farbe"), _
   Category("ColorControl")> _
   Property StartColor() As Color
        Get
            Return m_StartBrush.Color
        End Get
        Set(ByVal value As Color)
            ' Aktuelle und Start-Farbe festlegen
            m_StartBrush.Color = value
            m_ActiveBrush.Color = value
            ' Neu Zeichnen erzwingen
            Invalidate()
        End Set
    End Property
#End Region

#Region " Methoden "

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim g As Graphics = e.Graphics

        ' Flche weiss fllen
        g.Clear(Color.White)

        ' Zeichenflche mit aktiver Farbe fllen
        Dim InnerRect As New Rectangle(2, 2, Me.Width - 4, Me.Height - 4)
        g.FillRectangle(m_ActiveBrush, InnerRect)

        ' untere Hlfte der Zeichenflche mit Startfarbe fllen
        Dim HalbeHoehe As Integer = (Me.Height - 4) \ 2
        InnerRect = New Rectangle(2, 2 + HalbeHoehe, Me.Width - 4, Me.Height - 4 - HalbeHoehe)
        g.FillRectangle(m_StartBrush, InnerRect)

        ' Rahmen zeichnen
        ControlPaint.DrawVisualStyleBorder(g, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        ' quadratische Mae erzwingen
        Me.Width = Me.Height
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben der MouseDown-Methode
    ''' </summary>
    Private Sub BiColorBox_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        Dim HalbeHoehe As Integer = (Me.Height - 4) \ 2
        Dim InnerRect As New Rectangle(2, 2 + HalbeHoehe, Me.Width - 4, Me.Height - 4 - HalbeHoehe)
        If InnerRect.Contains(e.Location) Then
            ' wenn die untere Hlfte des Controls angeklickt wird, wird die angezeigte Farbe auf den
            ' Start-Wert zurck gesetzt (und das Changed-Event geliefert)
            If m_ActiveBrush.Color <> m_StartBrush.Color Then
                m_ActiveBrush.Color = m_StartBrush.Color
                ' Neu Zeichnen erzwingen
                Invalidate()
                ' nderungs-Event liefern
                RaiseEvent ColorChanged(Me)
            End If
        End If
    End Sub

#End Region

End Class
