Imports System.ComponentModel
''' <summary>
''' Ein Control, mit dem man bevorzugte Farben anzeigen und auswhlen kann
''' Man kann Farben hinzufgen oder entfernen oder mit der Maus neu sortieren
''' </summary>
Public Class ColorCollectorPanel
    Inherits Control
    Implements IColorControl

#Region " Private Felder / Events"
    ' Liste mit angezeigten Farben
    Private WithEvents m_Colors As New AddRemoveListe(Of Color)

    ' ausgewhlter Box-Index
    Private m_SelectedColorIndex As Integer = -1
    ' "gehooverter" Box-Index
    Private m_HooveredColorIndex As Integer = -1

    ' wenn ntig angezeigte ScrollBar
    Private WithEvents m_ScrollBar As VScrollBar

    ' Positionen fr Zeilen und Spalten
    Private m_LeftPositions As New List(Of Integer)
    Private m_TopPositions As New List(Of Integer)

    ' Info zum aktuell an der Mausposition vorhandenen Control-Inhalt
    Private m_MouseDownHoover As HooverInfo = Nothing

    ' Flag, die festlegt, ob intern gescrollt wird 
    ' (wenn False, wird beim ndern des ScrollBar-Wertes der Inhalt des Controls neu gezeichnet)
    Private m_InternScrolling As Boolean = False

    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged

    'Event, das geliefert wird, wenn eine neue ColorBox ausgewhlt wurde oder keine ColorBox mehr
    'ausgewhlt ist
    Public Event SelectionChanged()
#End Region

#Region " Konstruktoren "

    Public Sub New()
        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        m_ScrollBar = New VScrollBar
        Me.MinimumSize = New Size(50, 50)
        Me.Controls.Add(m_ScrollBar)
        m_ScrollBar.Visible = False

        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Liste mit Farbwerten
    ''' </summary>
    <Description("Liste mit Farbwerten"), _
   Category("ColorControl")> _
   Property Colors() As AddRemoveListe(Of Color)
        Get
            Return m_Colors
        End Get
        Set(ByVal value As AddRemoveListe(Of Color))
            If m_Colors IsNot value Then
                m_Colors = value
                SetSelectedColorIndex(-1)
                m_HooveredColorIndex = -1
                ' Zeilen- und Spaltenpositionen neu berechnen
                CalculatePositions()
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            If m_SelectedColorIndex = -1 Then
                Return Nothing
            Else
                Return m_Colors(m_SelectedColorIndex)
            End If
        End Get
        Set(ByVal value As System.Drawing.Color)
            Dim NewColorIndex As Integer = -1
            If Not (value = Nothing OrElse value = Drawing.Color.Transparent) Then
                For Each Col As Drawing.Color In m_Colors
                    If Col.R = value.R AndAlso Col.G = value.G AndAlso Col.B = value.B AndAlso Col.A = value.A Then
                        NewColorIndex = m_Colors.IndexOf(Col)
                        Exit For
                    End If
                Next
            End If


            If NewColorIndex = -1 Then
                If m_SelectedColorIndex <> -1 Then
                    SetSelectedColorIndex(-1)
                    m_HooveredColorIndex = -1
                    ' Neu Zeichnen erzwingen
                    Invalidate()
                End If
            Else
                If m_SelectedColorIndex <> NewColorIndex Then
                    SetSelectedColorIndex(NewColorIndex)
                    m_HooveredColorIndex = -1
                    ' Sicher stellen, dass die ausgewhlte Farbe auch im Control sichtbar ist,
                    ' falls ntig, die ScrollBar neu einstellen
                    If m_ScrollBar.Visible Then
                        m_InternScrolling = True
                        Dim Col As Integer = NewColorIndex Mod m_LeftPositions.Count
                        Dim Row As Integer = NewColorIndex \ m_LeftPositions.Count

                        Dim Rect As New Rectangle(m_LeftPositions(Col) + 6, m_TopPositions(Row) + 6, 24, 24)
                        Dim AcRect As New Rectangle(0, m_ScrollBar.Value, Me.Width, m_ScrollBar.Height)
                        If Not AcRect.Contains(Rect) Then
                            m_ScrollBar.Value = Math.Min(m_TopPositions(Row) + 6, m_ScrollBar.Maximum - m_ScrollBar.LargeChange)
                        End If


                        m_InternScrolling = False
                    End If
                    ' Neu Zeichnen erzwingen
                    Invalidate()
                End If
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Methode zum Neuberechnen der Positionen der ColorBoxen
    ''' </summary>
    Private Sub CalculatePositions()
        ' verhindern, dass nderung der ScrollBar zum Neuzeichnen des Controls fhrt
        m_InternScrolling = True

        m_LeftPositions.Clear()
        m_TopPositions.Clear()

        'wenn keine Farben in der Liste enthalten sind
        If m_Colors.Count = 0 Then
            m_ScrollBar.Visible = False
            m_ScrollBar.Value = 0
            m_InternScrolling = False
            Exit Sub
        End If

        'Abstand der ColorBoxen bestimmen
        Dim ControlWidth As Integer = Me.Width - 2
        Dim ControlHeight As Integer = Me.Height - 2

        Dim AvailableWidth As Integer = ControlWidth - 6
        Dim Columns As Integer = AvailableWidth \ 30
        If Columns = 0 Then Columns = 1

        Dim Rows As Integer = 1
        If Columns < m_Colors.Count Then
            Rows = (m_Colors.Count + Columns - 1) \ Columns
        End If

        Dim MaxHeight As Integer = Rows * 30 + 6
        If MaxHeight > ControlHeight Then
            ' ScrollBar ist sichtbar - es knnen eventuell nicht mehr 
            ' so viele ColorBoxen pro Zeile angezeigt werden
            AvailableWidth = ControlWidth - 6 - m_ScrollBar.Width
            Columns = AvailableWidth \ 30
            If Columns = 0 Then Columns = 1
            Rows = 1
            If Columns < m_Colors.Count Then
                Rows = (m_Colors.Count + Columns - 1) \ Columns
            End If
            MaxHeight = Rows * 30 + 6
        End If

        For I As Integer = 0 To Columns - 1
            m_LeftPositions.Add(I * 30 + 1)
        Next
        For I As Integer = 0 To Rows - 1
            m_TopPositions.Add(I * 30 + 1)
        Next


        If MaxHeight > ControlHeight Then
            ' wenn vertikal mehr Boxen anzuzeigen sind, als im Control mglich,
            ' ScrollBar-Position anzeigen:
            ' ScrollBar-Position festlegen
            m_ScrollBar.Location = New Point(Me.Width - m_ScrollBar.Width - 1, 1)
            m_ScrollBar.Height = ControlHeight

            Dim LargeChange As Integer = Math.Min(ControlHeight, MaxHeight - ControlHeight)
            m_ScrollBar.Maximum = MaxHeight - ControlHeight + LargeChange - 1
            m_ScrollBar.LargeChange = LargeChange

            m_ScrollBar.Visible = True
        Else
            ' ScrollBar verbergen
            m_ScrollBar.Value = 0
            m_ScrollBar.Visible = False
        End If
        m_InternScrolling = False
    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Positionen neu berechnen
        CalculatePositions()
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim Brsh As Brush
        Dim NumCols As Integer = m_LeftPositions.Count

        ' Fr alle in der Liste vorhandenen Farbwerte
        For I As Integer = 0 To m_Colors.Count - 1
            ' Spalte und Zeile fr Farb-Index ermitteln
            Dim Col As Integer = I Mod NumCols
            Dim Row As Integer = I \ NumCols

            ' Anzeige-Rechteck fr diese Farbe
            Dim Rect As New Rectangle(m_LeftPositions(Col) + 6, m_TopPositions(Row) + 6 - m_ScrollBar.Value, 24, 24)

            ' Nur wenn dieses Rechteck (abhngig von ScrollBar-Value) auf dem Control sichtbar ist
            If e.ClipRectangle.IntersectsWith(Rect) Then

                If m_SelectedColorIndex = I Then
                    ' Selektierte ColorBox: Rotes Rechteck zeichnen
                    Rect.Inflate(2, 2)
                    e.Graphics.FillRectangle(Brushes.Red, Rect)
                    Rect.Inflate(-3, -3)
                ElseIf m_HooveredColorIndex = I Then
                    ' Gehooverte ColorBox: Hoover-Rechteck zeichnen
                    Rect.Inflate(1, 1)
                    e.Graphics.FillRectangle(SystemBrushes.HotTrack, Rect)
                    Rect.Inflate(-2, -2)
                Else
                    ' Normale ColorBox: Schwarzes Rechteck zeichnen
                    e.Graphics.FillRectangle(Brushes.Black, Rect)
                    Rect.Inflate(-1, -1)
                End If

                ' weie Flche zeichnen
                e.Graphics.FillRectangle(Brushes.White, Rect)

                Rect.Inflate(-1, -1)
                Brsh = New SolidBrush(m_Colors(I))
                ' Farbflche zeichnen
                e.Graphics.FillRectangle(Brsh, Rect)
                Brsh.Dispose()

            End If
        Next

        ' wenn eine ColorBox innerhalb des Controls zu einem anderen Platz bewegt wird
        If m_MouseDownHoover IsNot Nothing AndAlso m_MouseDownHoover.Moving Then
            'eine selektierte ColorBox abhngig von der Maus-Position zeichnen
            Dim xRect As New Rectangle(m_MouseDownHoover.InsertColumn * 30 + 3, m_MouseDownHoover.InsertRow * 30 + 4 - m_ScrollBar.Value, 2, 30)
            If e.ClipRectangle.IntersectsWith(xRect) Then
                e.Graphics.FillRectangle(SystemBrushes.HotTrack, xRect)
            End If

            Dim Rect As New Rectangle(m_MouseDownHoover.acMousePoint.X + m_MouseDownHoover.Shift.Width + 6, m_MouseDownHoover.acMousePoint.Y + m_MouseDownHoover.Shift.Height + 6, 24, 24)
            Rect.Inflate(2, 2)
            e.Graphics.FillRectangle(Brushes.Red, Rect)
            Rect.Inflate(-3, -3)
            e.Graphics.FillRectangle(Brushes.White, Rect)
            Rect.Inflate(-1, -1)
            Brsh = New SolidBrush(m_Colors(m_MouseDownHoover.ColorIndex))
            e.Graphics.FillRectangle(Brsh, Rect)
            Brsh.Dispose()
        End If

        ' Rahmen fr das Control zeichnen
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' beim Scrollen der ScrollBar muss der Control-Inhalt neu gezeichnet werden
    ''' </summary>
    Private Sub m_ScrollBar_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles m_ScrollBar.Scroll
        ' wenn der Scrollbar-Wert durch eine interne Methode gendert wird, 
        ' an dieser Stelle nichts weiter tun
        If m_InternScrolling Then Exit Sub
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben der MouseDown-Methode, um gegebenenfalls eine neue ColorBox als Selected zu markieren
    ''' </summary>
    Private Sub ColorCollectorPanel_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        'nur bei linkem MouseButton reagieren
        If e.Button = Windows.Forms.MouseButtons.Left Then
            'HooverInfo (Zeile, Spalte, Farbe.. holen)
            m_MouseDownHoover = GetHoover(e.Location)

            'nur wenn wirklich eine ColorBox angeklickt wurde
            If m_MouseDownHoover IsNot Nothing Then
                'aktuelle Maus-Position an das Hoover-Info-Objekt bergeben
                m_MouseDownHoover.acMousePoint = e.Location

                ' nur wenn der Hoover-Farbindex wirklich neu ist
                If m_SelectedColorIndex <> m_MouseDownHoover.ColorIndex Then
                    SetSelectedColorIndex(m_MouseDownHoover.ColorIndex)
                    ' Farbnderungs-Event liefern
                    RaiseEvent ColorChanged(Me)
                    ' Neu Zeichnen erzwingen
                    Invalidate()
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' berschreiben der MouseMove-Methode, um gegebenenfalls eine ColorBox als Hoovered zu markieren
    ''' </summary>
    Private Sub ColorCollectorPanel_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If e.Button = Windows.Forms.MouseButtons.None Then
            ' wenn kein Button geklickt ist, Hoover ndern
            Dim Info As HooverInfo = GetHoover(e.Location)
            Dim NewColorIndex As Integer = -1
            If Info IsNot Nothing Then
                NewColorIndex = Info.ColorIndex
            End If
            ' wenn eine neue ColorBox ausgewhlt wurde
            If m_HooveredColorIndex <> NewColorIndex Then
                m_HooveredColorIndex = NewColorIndex
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        ElseIf e.Button = Windows.Forms.MouseButtons.Left Then
            ' wenn der linke Button bettigt wird, kann eine ColorBox bewegt werden

            ' nur wenn eine ColorBox durch Mausklick ausgewhlt wurde, 
            ' ist m_MouseDownHoover nicht Nothing
            If m_MouseDownHoover IsNot Nothing Then

                ' wenn schon bekannt ist, dass eine ColorBox bewegt wird
                If m_MouseDownHoover.Moving Then

                    ' wenn die Maus ber oder unter das Control bewegt wird, 
                    ' nach oben oder unten scrollen (wenn dies mglich ist)
                    If m_ScrollBar.Visible Then
                        If e.Location.Y < 0 Then
                            If Me.m_ScrollBar.Value > 0 Then
                                m_ScrollBar.Value -= 1
                            End If
                        ElseIf e.Location.Y > Me.Height Then
                            If Me.m_ScrollBar.Value < m_ScrollBar.Maximum - m_ScrollBar.LargeChange Then
                                m_ScrollBar.Value += 1
                            End If
                        End If
                    End If

                    'falls notwendig InsertIndex neu festlegen (zum Anzeigen der Einfgemarke)
                    If m_MouseDownHoover.acMousePoint <> e.Location Then
                        m_MouseDownHoover.acMousePoint = e.Location
                        SetInsertIndex(m_MouseDownHoover)
                        ' Neu Zeichnen erzwingen
                        Invalidate()
                    End If
                Else
                    Dim XDiff, YDiff As Integer
                    XDiff = Math.Max(e.X - m_MouseDownHoover.acMousePoint.X, m_MouseDownHoover.acMousePoint.X - e.X)
                    YDiff = Math.Max(e.Y - m_MouseDownHoover.acMousePoint.Y, m_MouseDownHoover.acMousePoint.Y - e.Y)
                    ' erst eine ColorBox bewegen, wenn die Maus mindestens vier Pixel vom Click-Punkt
                    ' entfernt ist
                    If XDiff > 4 OrElse YDiff > 4 Then
                        m_MouseDownHoover.Moving = True
                        m_MouseDownHoover.acMousePoint = e.Location
                        SetInsertIndex(m_MouseDownHoover)
                        ' Neu Zeichnen erzwingen
                        Invalidate()
                    End If
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' fr ein MouseDownHoover-Info-Objekt Einfge-Zeile, -Spalte und -Index festlegen
    ''' </summary>
    Private Sub SetInsertIndex(ByVal MouseDownHoover As HooverInfo)
        MouseDownHoover.InsertRow = Math.Max(0, Math.Min(m_TopPositions.Count - 1, (MouseDownHoover.acMousePoint.Y + m_ScrollBar.Value) \ 30))
        MouseDownHoover.InsertColumn = Math.Max(0, Math.Min(m_LeftPositions.Count, (MouseDownHoover.acMousePoint.X) \ 30))
        MouseDownHoover.InsertIndex = Math.Min(MouseDownHoover.InsertRow * m_LeftPositions.Count + MouseDownHoover.InsertColumn, m_Colors.Count)
        If MouseDownHoover.InsertRow = m_TopPositions.Count - 1 AndAlso MouseDownHoover.InsertIndex = m_Colors.Count Then
            MouseDownHoover.InsertColumn = MouseDownHoover.InsertIndex - m_LeftPositions.Count * (m_TopPositions.Count - 1)
        End If
    End Sub

    ''' <summary>
    ''' nach MouseUp wird eine bewegte ColorBox an die neue Position verschoben
    ''' </summary>
    Private Sub ColorCollectorPanel_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If m_MouseDownHoover IsNot Nothing Then

            ' nur wenn sich die Einfge-Position von der Originalposition der ColorBox unterscheidet
            If m_MouseDownHoover.Moving AndAlso m_MouseDownHoover.InsertIndex <> m_MouseDownHoover.ColorIndex AndAlso m_MouseDownHoover.InsertIndex <> m_MouseDownHoover.ColorIndex + 1 Then
                ' Farbe verschieben

                ' zu verschiebende Farbe an neuer Stelle einfgen
                m_Colors.Insert(m_MouseDownHoover.InsertIndex, m_Colors(m_MouseDownHoover.ColorIndex))

                ' Farbe von vorheriger Position entfernen
                If m_MouseDownHoover.InsertIndex > m_MouseDownHoover.ColorIndex Then
                    ' wenn der neue Farbindex grer als der alte ist
                    m_Colors.RemoveAt(m_MouseDownHoover.ColorIndex)
                    SetSelectedColorIndex(m_MouseDownHoover.InsertIndex - 1)
                Else
                    ' wenn der neue Farbindex kleiner als der alte ist
                    m_Colors.RemoveAt(m_MouseDownHoover.ColorIndex + 1)
                    SetSelectedColorIndex(m_MouseDownHoover.InsertIndex)
                End If
                m_HooveredColorIndex = -1
            End If
            ' an dieser Stelle wird das HooverInfo-Objekt nicht mehr gebraucht
            ' damit ist auch klar, dass die bewegte ColorBox nicht mehr gezeichnet wird
            m_MouseDownHoover = Nothing
            ' Neu zeichnen erzwingen
            Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' liefert ein Hoover-Info-Objekt fr eine Maus-Position
    ''' </summary>
    ''' <remarks>HooverInfo enthlt u.a. Zeile und Spalte sowie Farbindex und Shift fr eine ColorBox. Wenn sich die Maus nicht ber einer ColorBox bewindet, wird Nothing zurck gegeben.</remarks>
    Private Function GetHoover(ByVal MousePosition As Point) As HooverInfo
        'Ziel-Spalte
        Dim x As Integer = -1
        'Ziel-Zeile
        Dim y As Integer = -1

        'Shift: Abstand der Maus zur linken oberen Ecke der ColorBox
        Dim ShiftX, ShiftY As Integer

        Dim MP As New Point(MousePosition.X, MousePosition.Y + m_ScrollBar.Value)

        ' bestimmen, ob sich die x-Position in einer gltigen FarbBox-Zeile befindet
        For I As Integer = 0 To m_LeftPositions.Count - 1
            If m_LeftPositions(I) + 6 <= MP.X AndAlso m_LeftPositions(I) + 30 >= MP.X Then
                x = I
                ShiftX = m_LeftPositions(I) - MP.X
                Exit For
            End If
        Next
        ' wenn die Maus sich auerhalb einer gltigen Zeile befindet, wird Nothing zurck gegeben
        If x = -1 Then Return Nothing

        ' bestimmen, ob sich die y-Position in einer gltigen FarbBox-Spalte befindet
        For I As Integer = 0 To m_TopPositions.Count - 1
            If m_TopPositions(I) + 6 <= MP.Y AndAlso m_TopPositions(I) + 30 >= MP.Y Then
                y = I
                ShiftY = m_TopPositions(I) - MP.Y
                Exit For
            End If
        Next
        ' wenn die Maus auerhalb einer gltigen Spalte befindet, wird Nothing zurck gegeben
        If y = -1 Then Return Nothing

        ' Die letzte Zeile ist nicht immer mit ColorBoxen gefllt.
        ' wenn sich unter der Maus keine Box befindet, Nothing zurck geben
        Dim ColorIndex As Integer = y * m_LeftPositions.Count + x
        If Not (m_Colors.Count > ColorIndex) Then
            Return Nothing
        End If

        Return New HooverInfo(ColorIndex, y, x, New Size(ShiftX, ShiftY))

    End Function

    ''' <summary>
    ''' Interne Methode, die den SelectedIndex neu fest legt 
    ''' und falls ntig das SelectionChanged-Event liefert
    ''' </summary>
    Private Sub SetSelectedColorIndex(ByVal NewValue As Integer)
        If m_SelectedColorIndex <> NewValue Then
            m_SelectedColorIndex = NewValue
            RaiseEvent SelectionChanged()
        End If
    End Sub

    ''' <summary>
    ''' Interne Methode, die beim ndern der Inhalte der Farb-Liste den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Private Sub m_Colors_Changed() Handles m_Colors.Changed
        SetSelectedColorIndex(-1)
        m_HooveredColorIndex = -1
        ' Zeilen- und Spaltenpositionen neu berechnen
        CalculatePositions()
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub
#End Region

#Region " Interne Klassen "

    ''' <summary>
    ''' Interne Klasse, die Informationen zum Inhalt ber dem sich die Maus gerade befindet, liefert
    ''' </summary>
    Private Class HooverInfo

#Region " Private Felder "
        Private m_ColorIndex As Integer
        Private m_Row As Integer
        Private m_Column As Integer
        Private m_acMousePoint As Point
        Private m_Shift As Size
        Private m_Moving As Boolean = False
        Private m_InsertIndex As Integer = -1
        Private m_InsertRow As Integer = -1
        Private m_InserColumn As Integer = -1
#End Region

#Region " Properties "
        ''' <summary>
        ''' Einfge-Index
        ''' </summary>
        Property InsertIndex() As Integer
            Get
                Return m_InsertIndex
            End Get
            Set(ByVal value As Integer)
                m_InsertIndex = value
            End Set
        End Property
        ''' <summary>
        ''' Einfge-Zeile
        ''' </summary>
        Property InsertRow() As Integer
            Get
                Return m_InsertRow
            End Get
            Set(ByVal value As Integer)
                m_InsertRow = value
            End Set
        End Property
        ''' <summary>
        ''' Einfge-Spalte
        ''' </summary>
        Property InsertColumn() As Integer
            Get
                Return m_InserColumn
            End Get
            Set(ByVal value As Integer)
                m_InserColumn = value
            End Set
        End Property
        ''' <summary>
        ''' Index der ausgewhlten Farbe in der Farb-Liste
        ''' </summary>
        ReadOnly Property ColorIndex() As Integer
            Get
                Return m_ColorIndex
            End Get
        End Property
        ''' <summary>
        ''' Entfernung der Maus zum linken oberen Rand einer ColorBox (beim Klicken)
        ''' </summary>
        ReadOnly Property Shift() As Size
            Get
                Return m_Shift
            End Get
        End Property
        ''' <summary>
        ''' aktuelle Maus-Position im Control
        ''' </summary>
        Property acMousePoint() As Point
            Get
                Return m_acMousePoint
            End Get
            Set(ByVal value As Point)
                m_acMousePoint = value
            End Set
        End Property
        ''' <summary>
        ''' Flag, die festlegt, ob gerade eine ColorBox innerhalb des Controls bewegt wird
        ''' </summary>
        Property Moving() As Boolean
            Get
                Return m_Moving
            End Get
            Set(ByVal value As Boolean)
                m_Moving = value
            End Set
        End Property

#End Region

#Region " Konstruktor "
        ''' <summary>
        ''' Initialisiert ein HooverInfo-Objekt mit fest stehenden Werten
        ''' </summary>
        ''' <param name="ColorIndex">Farb-Index der ausgewhlten Farbe in der Control-eigenen Liste</param>
        ''' <param name="Row">Zeile</param>
        ''' <param name="Column">Spalte</param>
        ''' <param name="Shift">Abstand der Maus zur linken oberen Ecke einer ColorBox</param>
        Public Sub New(ByVal ColorIndex As Integer, ByVal Row As Integer, ByVal Column As Integer, ByVal Shift As Size)
            m_ColorIndex = ColorIndex
            m_Shift = Shift
            m_Row = Row
            m_Column = Column
        End Sub
#End Region

    End Class

#End Region

End Class

