Imports System.ComponentModel
''' <summary>
''' Ein Control, das fr eine Start-Farbe dazu passende Farben aus dem Farbkreis 
''' zur Auswahl anbietet
''' </summary>
Public Class ColorWheelPanel
    Inherits Control
    Implements IColorControl

#Region " Private Felder / Events"

    ' Array mit 4 x 4 Farben
    Private m_Colors(3, 3) As Color

    ' Basis-Farbe, fr die angezeigten Farbwerte im Array berechnet werden
    Private m_BaseColor As Color

    ' Breite und Hhe einer ColorBox
    Private m_BoxWidth As Integer

    ' Left- und Top-Positionen der der ColorBoxen
    Private m_Positions(3) As Integer

    ' Zeile/Spalte fr aktuell ausgewhlte ColorBox
    Private m_SelectedPoint As New Point(-1, -1)
    ' Zeile/Spalte fr aktuell gehooverte ColorBox
    Private m_HooveredPoint As New Point(-1, -1)

    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged

#End Region

#Region " Konstruktoren "

    Public Sub New()
        ' Wesentliche Eigenschaften auf Start-Werte festlegen

        Me.MinimumSize = New Size(128, 128)
        Me.Size = New Size(128, 128)
        SetColors(Color.Red)

        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            ' um das Control im Windows Forms Designer problemlos anzuzeigen, 
            ' muss hier immer eine Farbe aus dem m_Colors-Array geliefert werden
            If m_SelectedPoint.X = -1 OrElse m_SelectedPoint.Y = -1 Then
                Return m_Colors(0, 0)
            Else
                Return m_Colors(m_SelectedPoint.Y, m_SelectedPoint.X)
            End If
        End Get
        Set(ByVal value As System.Drawing.Color)
            ' wenn die Farbe der BasisFarbe im Control entspricht
            If m_BaseColor.R = value.R AndAlso m_BaseColor.G = value.G AndAlso m_BaseColor.B = value.B AndAlso m_BaseColor.A = value.A Then
                m_SelectedPoint = New Point(0, 0)
                m_HooveredPoint = New Point(-1, -1)
                ' Neu Zeichnen erzwingen
                Invalidate()
            Else
                'Farb-Werte neu berechnen
                SetColors(value)
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' berechnet die angezeigten Farb-Werte auf Grundlage einer Basis-Farbe neu
    ''' </summary>
    ''' <param name="BaseColor">Basis-Farbe</param>
    Private Sub SetColors(ByVal BaseColor As Color)
        ' Hoover und Selection auf Null setzen
        m_SelectedPoint = New Point(-1, -1)
        m_HooveredPoint = New Point(-1, -1)
        m_BaseColor = BaseColor


        Dim Cols As New List(Of Color)
        Dim HSL As New HSLColor(BaseColor)
        Cols.Add(BaseColor)
        Dim TmpH As Double = HSL.Hue
        For i As Integer = 0 To 10
            'Hue (Farbton) um jeweils 30 Grad verschieben
            TmpH += 1.0R / 12.0R
            If TmpH > 1.0R Then
                TmpH -= 1.0R
            ElseIf TmpH < 0.0R Then
                TmpH += 1.0R
            End If
            HSL.Hue = TmpH
            Cols.Add(HSL.ToRGB)
        Next

        Dim direction As Double
        For i As Integer = 1 To 4
            If i = 1 Then
                If HSL.Lightness < 0.6R Then
                    direction = 1.0R
                Else
                    direction = -1.0R
                End If
            ElseIf i = 3 Then
                HSL = New HSLColor(BaseColor)
                TmpH = HSL.Hue
                TmpH += 0.5R
                If TmpH > 1.0R Then
                    TmpH -= 1.0R
                ElseIf TmpH < 0.0R Then
                    TmpH += 1.0R
                End If
                HSL.Hue = TmpH
            End If


            Dim TmpL As Double = HSL.Lightness + direction * 0.125R
            'Luminanz um 8 Prozent ndern
            If TmpL > 1.0R Then
                TmpL -= 1.0R
            ElseIf TmpL < 0.0R Then
                TmpL += 1.0R
            End If
            HSL.Lightness = TmpL

            Cols.Add(HSL.ToRGB)
        Next i

        ' Farben im Array durch gerade ermittelte Farben ersetzen
        m_Colors(0, 0) = Cols(0)
        m_Colors(0, 1) = Cols(1)
        m_Colors(0, 2) = Cols(2)
        m_Colors(0, 3) = Cols(3)
        m_Colors(1, 3) = Cols(4)
        m_Colors(2, 3) = Cols(5)
        m_Colors(3, 3) = Cols(6)
        m_Colors(3, 2) = Cols(7)
        m_Colors(3, 1) = Cols(8)
        m_Colors(3, 0) = Cols(9)
        m_Colors(2, 0) = Cols(10)
        m_Colors(1, 0) = Cols(11)
        m_Colors(1, 1) = Cols(12)
        m_Colors(1, 2) = Cols(13)
        m_Colors(2, 2) = Cols(14)
        m_Colors(2, 1) = Cols(15)
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim Brsh As Brush

        ' Fr alle Farbwerte im Array
        For I As Integer = 0 To 3
            For J As Integer = 0 To 3
                'Anzeige-Rechteck ermitteln
                Dim Rect As New Rectangle(m_Positions(I), m_Positions(J), m_BoxWidth, m_BoxWidth)

                If m_SelectedPoint.X = I AndAlso m_SelectedPoint.Y = J Then
                    ' Wenn die Box selektiert ist, roter Rahmen
                    Rect.Inflate(2, 2)
                    e.Graphics.FillRectangle(Brushes.Red, Rect)
                    Rect.Inflate(-3, -3)
                ElseIf m_HooveredPoint.X = I AndAlso m_HooveredPoint.Y = J Then
                    ' Wenn die Box gehoovert ist, Hoover-Rahmen
                    Rect.Inflate(1, 1)
                    e.Graphics.FillRectangle(SystemBrushes.HotTrack, Rect)
                    Rect.Inflate(-2, -2)
                Else
                    ' ansonsten schwarzer Rahmen
                    e.Graphics.FillRectangle(Brushes.Black, Rect)
                    Rect.Inflate(-1, -1)
                End If


                ' weie Flche zeichnen
                e.Graphics.FillRectangle(Brushes.White, Rect)
                Rect.Inflate(-1, -1)
                Brsh = New SolidBrush(m_Colors(J, I))
                'farbigen Inhalt zeichnen
                e.Graphics.FillRectangle(Brsh, Rect)
                Brsh.Dispose()
            Next
        Next

        'Rahmen des Controls zeichnen
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Hhe muss gleich Breite sein
        If Me.Width > Me.Height Then
            Me.Height = Me.Width
        Else
            Me.Width = Me.Height
        End If
        ' Breite und Abstand der ColorBoxen neu berechnen
        RecalculateBoxes()
    End Sub

    ''' <summary>
    ''' Breite und Abstand der ColorBoxen neu festlegen
    ''' </summary>
    Private Sub RecalculateBoxes()
        ' 4 * BoxWidth + 5 x BoxDistance = Width

        'BoxWidth = 30
        'BoxDistance = 5
        'fr BoxWidth kann BoxDistance * 6 eingesetzt werden

        '-> 4 * 6 * BoxDistance + 5 x BoxDistance = Width
        '24 * BoxDistance + 5 x BoxDistance = Width
        '29 x BoxDistance = Width

        Dim BoxDistance As Integer = (Me.Width - 2) \ 29
        m_BoxWidth = BoxDistance * 6

        Dim RestWidth As Integer = Me.Width - 2 - m_BoxWidth * 4
        BoxDistance = RestWidth \ 5

        Dim LineWidth As Integer = BoxDistance * 3 + m_BoxWidth * 4
        Dim BorderWith As Integer = (Me.Width - LineWidth) \ 2


        For I As Integer = 0 To 3
            m_Positions(I) = BorderWith + I * (m_BoxWidth + BoxDistance)
        Next
    End Sub

    ''' <summary>
    ''' berschreiben der MouseDown-Methode, um gegebenenfalls eine ColorBox als Selected zu markieren
    ''' </summary>
    Private Sub ColorWheelPanel_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Left Then
            ' nur bei aktivem linkem Mouse-Button
            ' Hoover (Zeile/Spalte) ermitteln, an dem sich die Maus gerade befindet
            Dim Hoover As Point = GetHoover(e.Location)
            ' wenn die Maus keine ColorBox berhrt, ist nichts weiter zu tun
            If Hoover.X = -1 AndAlso Hoover.Y = -1 Then Exit Sub
            If m_SelectedPoint <> Hoover Then
                ' nur wenn eine neue ColorBox durch die Maus "angeklickt" wurde
                m_SelectedPoint = Hoover
                ' nderungs-Event liefern
                RaiseEvent ColorChanged(Me)
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End If
    End Sub

    ''' <summary>
    ''' berschreiben der MouseMove-Methode, um gegebenenfalls eine ColorBox als Hoovered zu markieren
    ''' </summary>
    Private Sub ColorWheelPanel_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        'Hoovern funktioniert nur, wenn kein Mouse-Button bettigt ist
        If e.Button = Windows.Forms.MouseButtons.None Then
            ' Hoover (Zeile/Spalte) ermitteln, an dem sich die Maus gerade befindet
            Dim Hoover As Point = GetHoover(e.Location)
            If m_HooveredPoint <> Hoover Then
                ' HooveredPoint neu festlegen
                m_HooveredPoint = Hoover
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End If
    End Sub

    ''' <summary>
    ''' liefert eine Point-Struktur, die Zeile und Spalte fr eine ColorBox enthlt, fr eine gegebene Maus-Position
    ''' </summary>
    Private Function GetHoover(ByVal MousePosition As Point) As Point

        'Ziel-Spalte
        Dim x As Integer = -1
        'Ziel-Zeile
        Dim y As Integer = -1

        For I As Integer = 0 To 3
            If m_Positions(I) <= MousePosition.X AndAlso m_Positions(I) + m_BoxWidth >= MousePosition.X Then
                x = I
                Exit For
            End If
        Next
        For I As Integer = 0 To 3
            If m_Positions(I) <= MousePosition.Y AndAlso m_Positions(I) + m_BoxWidth >= MousePosition.Y Then
                y = I
                Exit For
            End If
        Next

        ' Wenn Ziel-Zeile oder Ziel-Spalte nicht ermittelt werden konnten, befinde sich die Maus nicht
        ' ber einer ColorBox
        If x <> -1 AndAlso y <> -1 Then
            Return New Point(x, y)
        Else
            Return New Point(-1, -1)
        End If
    End Function

#End Region

End Class
