Imports System.ComponentModel
''' <summary>
''' Ein Control zum Anzeigen eines Farbgradienten in einer Box
''' </summary>
Public Class GradientColorBox
    Inherits Control

#Region " Private Felder / Events"
    Private m_Color As GradientColor
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Initialisierung  wesentlicher Eigenschaften dieses Controls
        Me.MinimumSize = New Size(50, 50)
        m_Color = New GradientColor(Color.Red, Color.White, 0, 0)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Gradient-Farbe, die angezeigt wird
    ''' </summary>
    WriteOnly Property GradientColor() As GradientColor
        Set(ByVal value As GradientColor)
            If value IsNot Nothing Then
                m_Color = New GradientColor(value)
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim g As Graphics = e.Graphics

        ' Flche weiss fllen
        g.Clear(Color.White)

        ' Gradient zeichnen
        Dim InnerRect As New Rectangle(2, 2, Me.Width - 4, Me.Height - 4)
        m_Color.Draw(e.Graphics, InnerRect)

        ' Rahmen zeichnen
        ControlPaint.DrawVisualStyleBorder(g, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub
    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        ' Control muss quadratisch sein
        Me.Width = Me.Height
        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub
#End Region

End Class
