Imports System.ComponentModel
''' <summary>
''' Ein Control, mit dem man einen CMYK-Farb-Wert anzeigen oder festlegen kann
''' </summary>
Public Class CMYKControl
    Implements IColorControl

#Region " Private Felder / Events"
    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
    Private m_Color As Color = Drawing.Color.Black
#End Region

#Region " Konstruktoren "
    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.Color = Drawing.Color.Red
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If m_Color <> value Then
                m_Color = value
                Dim CMYK As New CMYKColor(m_Color)
                Me.TextBoxCyan.ValidatedValue = (CInt(Math.Round(CMYK.Cyan * 100.0R))).ToString
                Me.TextBoxMagenta.ValidatedValue = (CInt(Math.Round(CMYK.Magenta * 100.0R))).ToString
                Me.TextBoxYellow.ValidatedValue = (CInt(Math.Round(CMYK.Yellow * 100.0R))).ToString
                Me.TextBoxKey.ValidatedValue = (CInt(Math.Round(CMYK.Black * 100.0R))).ToString
            End If
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' Interne Methode, die nach nderung in einer abhngigen ValidatedTextbox 
    ''' den eigenen Farbwert neu einstellt und das Changed-Event liefert
    ''' </summary>
    Private Sub NumericTextBox_ValidatedValue_Changed() Handles TextBoxCyan.ValidatedValue_Changed, TextBoxMagenta.ValidatedValue_Changed, TextBoxYellow.ValidatedValue_Changed, TextBoxKey.ValidatedValue_Changed
        Dim Cyan As Double = CDbl(TextBoxCyan.ValidatedValue) / 100.0R
        Dim Magenta As Double = CDbl(TextBoxMagenta.ValidatedValue) / 100.0R
        Dim Yellow As Double = CDbl(TextBoxYellow.ValidatedValue) / 100.0R
        Dim K As Double = CDbl(TextBoxKey.ValidatedValue) / 100.0R
        Dim CMYK As New CMYKColor(Cyan, Magenta, Yellow, K)

        Dim NewColor As Color = CMYK.ToRGB
        If m_Color <> NewColor Then
            m_Color = NewColor
            RaiseEvent ColorChanged(Me)
        End If
    End Sub
#End Region

End Class
