Imports System.ComponentModel
''' <summary>
''' Ein TextBox-Control, das nur Eingabe von Integer-Werten in einem festgelegten Wertebereich erlaubt
''' </summary>
Public Class NumericTextBox
    Inherits ValidatedTextBox

#Region " Private Felder"
    Private m_MinValue As Integer = 0
    Private m_MaxValue As Integer = 100
#End Region

#Region " Properties "
    ''' <summary>
    ''' gltiger Minimalwert
    ''' </summary>
    Property MinValue() As Integer
        Get
            Return m_MinValue
        End Get
        Set(ByVal value As Integer)
            If m_MinValue <> value Then
                m_MinValue = value
                m_MaxValue = Math.Max(m_MaxValue, m_MinValue)
            End If
        End Set
    End Property

    ''' <summary>
    ''' gltiger Maximalwert
    ''' </summary>
    Property MaxValue() As Integer
        Get
            Return m_MaxValue
        End Get
        Set(ByVal value As Integer)
            If m_MaxValue <> value Then
                m_MaxValue = value
                m_MinValue = Math.Min(m_MaxValue, m_MinValue)
            End If
        End Set
    End Property

#End Region

#Region " berschriebene Properties "
    ''' <summary>
    ''' Valider Startwert
    ''' </summary>
    Public Overrides ReadOnly Property AllowedChars() As String
        Get
            Return "1234567890"
        End Get
    End Property

    ''' <summary>
    ''' erlaubte Zeichen
    ''' </summary>
    Public Overrides ReadOnly Property BaseValue() As String
        Get
            Return "0"
        End Get
    End Property
#End Region

#Region " berschriebene Methoden "
    ''' <summary>
    ''' Methode, die fr den in der TextBox enthaltenen Text ein ValidatedValue-Objekt liefert
    ''' </summary>
    ''' <param name="TextBoxOriginal">Inhalt der TextBox vor Validierung</param>
    ''' <param name="LastValidatedValue">letzter gltiger (validierter) Inhalt der TextBox</param>
    ''' <remarks>ValidatedValue liefert Value(String) und gegebenenfalls Fehlermeldung</remarks>
    Public Overrides Function GetValideText(ByVal TextBoxOriginal As String, ByVal LastValidatedValue As String) As ValidatedValue
        Dim ResultValue As String = ""
        Dim HasError As Boolean = True

        Try
            Dim TmpVal As Integer = CInt(TextBoxOriginal)
            If TmpVal >= MinValue Then 'AndAlso TmpVal <= MaxValue Then
                If TmpVal <= MaxValue Then
                    ResultValue = TmpVal.ToString
                    HasError = False
                Else
                    ResultValue = m_MaxValue.ToString
                End If
            Else
                ResultValue = m_MinValue.ToString
            End If
        Catch ex As Exception
            ResultValue = LastValidatedValue
        End Try


        If HasError Then
            Return New ValidatedValue(ResultValue, "Bitte geben Sie eine Zahl zwischen " & m_MinValue.ToString & " und " & m_MaxValue.ToString & " ein." & Chr(10) & "Der nchstmgliche Wert wird eingesetzt.")
        Else
            Return New ValidatedValue(ResultValue, "")
        End If
    End Function
#End Region

End Class
