Imports System.ComponentModel
''' <summary>
''' Basisklasse fr die hier verwendeten validierten TextBox-Controls,
''' die nur Eingabe vorher festgelegter Zeichen erlauben
''' </summary>
Public MustInherit Class ValidatedTextBox
    Inherits TextBox

#Region " Private Felder / Events"
    ' erlaubte Funktionstasten
    Private Shared m_allowedActions As List(Of System.Char)

    'Flag zum Festlegen, ob das Control das ValidatedValue_Changed Event liefert
    Private m_SuppressEvents As Boolean = False

    'Aktuell valider Wert
    Private m_ValidatedValue As String = ""

    'Event, das geliefert wird, wenn ein neuer valider Wert nach Eingabe in der TextBox zur Verfgung steht
    Public Event ValidatedValue_Changed()
#End Region

#Region " Konstruktoren "
    Shared Sub New()
        m_allowedActions = New List(Of System.Char)
        m_allowedActions.Add(System.Convert.ToChar(Keys.Back))
        m_allowedActions.Add(System.Convert.ToChar(Keys.Enter))
        m_allowedActions.Add(System.Convert.ToChar(Keys.Tab))
        m_allowedActions.Add(System.Convert.ToChar(Keys.Cancel))
        m_allowedActions.Add(System.Convert.ToChar(Keys.Delete))
        m_allowedActions.Add(System.Convert.ToChar(Keys.Escape))
    End Sub
    Public Sub New()
        ' Wesentliche Eigenschaften auf Start-Werte festlegen
        m_ValidatedValue = BaseValue
        m_SuppressEvents = True
        Me.Text = m_ValidatedValue.ToString
        m_SuppressEvents = False
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Valider String, der in der TextBox angezeigt wird
    ''' </summary>
    Property ValidatedValue() As String
        Get
            Return m_ValidatedValue
        End Get
        Set(ByVal value As String)
            If m_ValidatedValue <> value Then
                m_ValidatedValue = value
                m_SuppressEvents = True
                Me.Text = m_ValidatedValue.ToString
                m_SuppressEvents = False
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' tritt ein, wenn der Text in der TextBox gendert wurde
    ''' </summary>
    Private Sub ValidatedTextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.TextChanged
        If m_SuppressEvents Then Exit Sub

        'hier eventuell neu ValidatedValue setzen und Changed-Event feuern
        Dim Val As ValidatedValue = GetValideText(Me.Text, m_ValidatedValue)
        If Val.Message = "" Then
            If m_ValidatedValue <> Val.Value Then
                m_ValidatedValue = Val.Value
                RaiseEvent ValidatedValue_Changed()
            End If
        End If
    End Sub

    ''' <summary>
    ''' tritt ein, wenn die TextBox den Focus verliert
    ''' </summary>
    Private Sub ValidatedTextBox_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.Validating
        'hier Schlussberprfung vornehmen, Korrekturmeldung und Formatierung
        Dim Val As ValidatedValue = GetValideText(Me.Text, m_ValidatedValue)
        m_SuppressEvents = True
        Me.Text = Val.Value
        m_SuppressEvents = False
        If m_ValidatedValue <> Val.Value Then
            m_ValidatedValue = Val.Value
            RaiseEvent ValidatedValue_Changed()
        End If

        If Val.Message <> "" Then
            MessageBox.Show(Val.Message)
            e.Cancel = True
            Me.SelectAll()
        End If
    End Sub

    ''' <summary>
    ''' tritt bei einem Tastendruck in der TextBox ein 
    ''' </summary>
    Private Sub ValidatedTextBox_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Me.KeyPress
        If (Not m_allowedActions.Contains(e.KeyChar)) AndAlso (Not AllowedChars.Contains(e.KeyChar)) Then
            e.Handled = True
        End If
    End Sub

#End Region

#Region " Methoden und Properties, die berschrieben werden mssen "

    ''' <summary>
    ''' Muss berschrieben werden: erlaubte Zeichen
    ''' </summary>
    MustOverride ReadOnly Property AllowedChars() As String

    ''' <summary>
    ''' Muss berschrieben werden: Valider Startwert
    ''' </summary>
    MustOverride ReadOnly Property BaseValue() As String

    ''' <summary>
    ''' Muss berschrieben werden: Methode, die fr einen in der TextBox enthaltenen Text ein ValidatedValue-Objekt liefert
    ''' </summary>
    ''' <param name="TextBoxOriginal">Inhalt der TextBox vor Validierung</param>
    ''' <param name="LastValidatedValue">letzter gltiger (validierter) Inhalt der TextBox</param>
    ''' <remarks>ValidatedValue liefert Value(String) und gegebenenfalls Fehlermeldung</remarks>
    MustOverride Function GetValideText(ByVal TextBoxOriginal As String, ByVal LastValidatedValue As String) As ValidatedValue

#End Region

End Class
