Imports System.ComponentModel
''' <summary>
''' Demo-Form mit verschiedenen Farbanzeige-Controls
''' </summary>
Public Class frm_ColorPicker

#Region " Private Felder / Events"

    ' aktuelle Farbe
    Private m_Color As Color

    ' Liste mit Farbanzeige-Controls
    Private m_Controls As New List(Of IColorControl)

#End Region

#Region " Konstruktoren "

    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.

        ' Farb-Liste fr das ColorCollectorPanel festlegen
        Me.ColorCollectorPanel1.Colors = SharedColors.AllColors

        ' eigene Start-Farbe festlegen
        m_Color = Drawing.Color.FromArgb(255, 0, 0)

        ' Farbanzeige-Controls in Liste sammeln
        m_Controls.Add(Me.BiColorBox1)
        m_Controls.Add(Me.ColorCollectorPanel1)
        m_Controls.Add(Me.AdobeColorPicker1)
        m_Controls.Add(Me.ColorWheelPanel1)
        m_Controls.Add(Me.RgbControl1)
        m_Controls.Add(Me.HsbControl1)
        m_Controls.Add(Me.CmykControl1)
        m_Controls.Add(Me.HexColorControl1)
        m_Controls.Add(Me.EyeDropperButton1)

        ' fr alle Farbanzeige-Controls 
        For Each Ctl As IColorControl In m_Controls
            ' Startfarbe festlegen
            Ctl.Color = m_Color
            ' Eventhandler fr ColorChanged-Ereignis festlegen
            AddHandler Ctl.ColorChanged, AddressOf ColCtl_ColorChanged
        Next
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' aktuelle Farbe
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Property Color() As Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As Color)
            If value.R <> m_Color.R OrElse value.G <> m_Color.G OrElse value.B <> m_Color.B Then
                m_Color = value
                ' alle Farbanzeige-Controls in der Liste erhalten diese Farbe
                For Each Ctl As IColorControl In m_Controls
                    Ctl.Color = m_Color
                Next
                ' Fr die BiColorbox muss eine Start-Farbe festgelegt werden
                Me.BiColorBox1.StartColor = m_Color
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' aktiviert / deaktiviert den Button zum Entfernen einer Farbe aus dem ColorCollectorPanel
    ''' </summary>
    Private Sub ColorCollectorPanel1_SelectionChanged() Handles ColorCollectorPanel1.SelectionChanged
        If ColorCollectorPanel1.Color = Nothing Then
            Me.ButtonRemoveColor.Enabled = False
        Else
            Me.ButtonRemoveColor.Enabled = True
        End If
    End Sub

    ''' <summary>
    ''' Entfernen der ausgewhlten Farbe aus dem ColorCollectorPanel
    ''' </summary>
    Private Sub ButtonRemoveColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonRemoveColor.Click
        ColorCollectorPanel1.Colors.Remove(ColorCollectorPanel1.Color)
    End Sub

    ''' <summary>
    ''' Hinzufgen der ausgewhlten Farbe zum ColorCollectorPanel
    ''' </summary>
    Private Sub ButtonAddColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAddColor.Click
        If Not ColorCollectorPanel1.Colors.Contains(Me.BiColorBox1.Color) Then
            ColorCollectorPanel1.Colors.Add(Me.BiColorBox1.Color)
            ColorCollectorPanel1.Color = Me.BiColorBox1.Color
        Else
            ColorCollectorPanel1.Color = Me.BiColorBox1.Color
        End If
    End Sub

    ''' <summary>
    ''' Eventhandler fr das ColorChanged-Event aller Farbanzeige-Controls
    ''' </summary>
    ''' <param name="Sender">Control, das das aktuelle Event liefert </param>
    Private Sub ColCtl_ColorChanged(ByVal Sender As IColorControl)
        If Sender.Color <> Nothing Then
            ' wenn wirklich eine neue Farbe ausgewhlt wurde
            Dim NewColor As Color = Sender.Color
            If NewColor.R <> m_Color.R OrElse NewColor.G <> m_Color.G OrElse NewColor.B <> m_Color.B Then
                'Interne Farbe festlegen
                m_Color = NewColor
                'alle Farbanzeige-Controls auer Sender sollen die aktuelle Farbe anzeigen
                For Each Ctl As IColorControl In m_Controls
                    If Not Ctl Is Sender Then
                        Ctl.Color = m_Color
                    End If
                Next
            End If

        End If
    End Sub

#End Region

End Class